package com.bocloud.sms.booter.config;

import java.util.List;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import com.bocloud.sms.entity.ApiToken;
import com.bocloud.sms.entity.Role;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.entity.UserGroup;
import com.bocloud.sms.repository.ApiTokenRepository;
import com.bocloud.sms.repository.RoleRepository;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserGroupRepository;
import com.bocloud.sms.repository.UserRepository;
import com.megatron.common.model.RequestContext;

import lombok.extern.slf4j.Slf4j;

/**
 * @author tyl
 * @since 2020/3/25 周三 14:06
 */
@Component
@Slf4j
public class AccountCache implements InitializingBean {

    private final StringRedisTemplate redisTemplate;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final RoleRepository roleRepository;
    private final UserGroupRepository userGroupRepository;
    private final ApiTokenRepository apiTokenRepository;
    private final ThreadPoolTaskExecutor taskExecutor;

    public AccountCache(StringRedisTemplate redisTemplate, TenantRepository tenantRepository,
        UserRepository userRepository, RoleRepository roleRepository, UserGroupRepository userGroupRepository,
        ApiTokenRepository apiTokenRepository, ThreadPoolTaskExecutor taskExecutor) {
        this.redisTemplate = redisTemplate;
        this.tenantRepository = tenantRepository;
        this.userRepository = userRepository;
        this.roleRepository = roleRepository;
        this.userGroupRepository = userGroupRepository;
        this.apiTokenRepository = apiTokenRepository;
        this.taskExecutor = taskExecutor;
    }

    public void putAccountCache(RequestContext.Catalog catalog, Long id, String account) {
        String key = catalog.name() + "_" + id;
        this.redisTemplate.opsForValue().set(key, account);
    }

    public void putNameCache(RequestContext.Catalog catalog, Long id, String name) {
        String key = catalog.name() + "_Name_" + id;
        this.redisTemplate.opsForValue().set(key, name);
    }

    public void putRoleCache(Long id, String name) {
        this.redisTemplate.opsForValue().set("Role_Name_" + id, name);
    }

    public void putUserGroupCache(Long id, String name) {
        this.redisTemplate.opsForValue().set("Group_Name_" + id, name);
    }

    public void putApiTokenCache(String token, String apis) {
        this.redisTemplate.opsForValue().set("Api_Token_" + token, apis);
    }

    /**
     * 获取缓存中用户账号
     *
     * @param id
     *            用户id
     * @return
     */
    public String getUserAccount(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + id);
    }

    /**
     * 获取缓存中用户名称
     *
     * @param id
     *            用户id
     * @return
     */
    public String getUserName(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_Name_" + id);
    }

    /**
     * 获取缓存中租户账号
     *
     * @param id
     *            租户id
     * @return
     */
    public String getTenantAccount(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_" + id);
    }

    /**
     * 获取缓存中租户名称
     *
     * @param id
     *            租户id
     * @return
     */
    public String getTenantName(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_Name_" + id);
    }

    /**
     * 获取缓存中角色名称
     *
     * @param id
     *            租户id
     * @return
     */
    public String getRoleName(Long id) {
        return redisTemplate.opsForValue().get("Role_Name_" + id);
    }

    /**
     * 获取缓存中用户组名称
     *
     * @param id
     *            租户id
     * @return
     */
    public String getUserGroupName(Long id) {
        return redisTemplate.opsForValue().get("Group_Name_" + id);
    }

    public String getApiToken(String api) {
        return redisTemplate.opsForValue().get("Api_Token_" + api);
    }

    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(this::addTenantCache);
        taskExecutor.execute(this::addUserCache);
        taskExecutor.execute(this::addRoleCache);
        taskExecutor.execute(this::addUserGroupCache);
        taskExecutor.execute(this::addApiTokenCache);
    }

    public void addTenantCache() {
        try {
            List<Tenant> tenants = tenantRepository.listAll();
            // 缓存租户账号和租户名
            for (Tenant tenant : tenants) {
                if (!tenant.getDeleted()) {
                    putAccountCache(RequestContext.Catalog.Tenant, tenant.getId(), tenant.getAccount());
                }
                putNameCache(RequestContext.Catalog.Tenant, tenant.getId(), tenant.getName());
            }
        } catch (Exception e) {
            log.error("put tenant cache error,", e);
        }
    }

    public void addUserCache() {
        try {
            List<User> users = userRepository.listAll();
            // 缓存用户账号和用户名
            for (User user : users) {
                if (!user.getDeleted()) {
                    putAccountCache(RequestContext.Catalog.User, user.getId(), user.getAccount());
                }
                putNameCache(RequestContext.Catalog.User, user.getId(), user.getName());
            }
        } catch (Exception e) {
            log.error("put user cache error,", e);
        }
    }

    public void addRoleCache() {
        try {
            List<Role> roles = roleRepository.list();
            // 缓存角色名
            for (Role role : roles) {
                putRoleCache(role.getId(), role.getName());
            }
        } catch (Exception e) {
            log.error("put role cache error,", e);
        }
    }

    public void addUserGroupCache() {
        try {
            List<UserGroup> groups = userGroupRepository.list();
            // 缓存角色名
            for (UserGroup group : groups) {
                putUserGroupCache(group.getId(), group.getName());
            }
        } catch (Exception e) {
            log.error("put user group cache error,", e);
        }
    }

    public void addApiTokenCache() {
        try {
            List<ApiToken> tokens = apiTokenRepository.listAll();
            // 缓存角色名
            for (ApiToken token : tokens) {
                if (!token.getEnabled()) {
                    continue;
                }
                putApiTokenCache(token.getToken(), token.getApis());
            }
        } catch (Exception e) {
            log.error("put api token cache error,", e);
        }
    }

}
