package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.ManagerService;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.interfaces.UserPermissionService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.*;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.util.List;
import java.util.Map;

/**
 * @author tyl
 */
@RequiredArgsConstructor
@RestController
@RequestMapping("/v1/users")
@Tag(name = "用户管理")
public class UserController {

    private final UserService userService;
    private final UserPermissionService userPermissionService;
    private final TenantService tenantService;
    private final ManagerService managerService;

    /**
     * 获取用户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Operation(summary = "查询用户列表")
    @GetMapping
    public GeneralResult<GridBean<User>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.list(pager, requestContext);
    }

    /**
     * 创建用户
     *
     * @param userBean
     * @param requestContext
     * @return
     */
    @Operation(summary = "创建用户")
    @PostMapping
    public Result create(@RequestBody UserBean userBean, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.create(userBean, requestContext);
    }

    /**
     * 修改用户
     *
     * @param userBean
     * @param requestContext
     * @return
     */
    @Operation(summary = "用户修改")
    @PutMapping("/{id}")
    public Result modify(@RequestBody UserBean userBean, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                         @PathVariable(value = Common.ID) Long id) {
        return userService.modify(id, userBean, requestContext);
    }

    /**
     * 移除用户
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Operation(summary = "删除用户")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.remove(id, requestContext);
    }

    /**
     * 查看用户详细信息
     *
     * @param id
     * @return
     */
    @Operation(summary = "用户详情")
    @GetMapping("/{id}")
    public GeneralResult<User> detail(@PathVariable(value = Common.ID) Long id) {
        return userService.detail(id);
    }


    /**
     * 查询用户账号是否存在
     *
     * @param account
     * @return
     */
    @Operation(summary = "查询用户账号是否存在")
    @GetMapping("/{account}/exist")
    public Result exist(@PathVariable(value = Common.ACCOUNT) String account) {
        return userService.exist(account);
    }

    /**
     * 查看租户详情
     *
     * @param requestContext
     * @return
     */
    @Operation(summary = "用户详情")
    @GetMapping("/detail")
    public GeneralResult<User> detail(@Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.detail(requestContext.getTarget());
    }


    /**
     * 修改用户状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     */
    @Operation(summary = "修改用户状态")
    @PatchMapping("/{id}")
    public Result updateStatus(@PathVariable(value = Common.ID) Long id,
                               @RequestBody ActionModel action,
                               @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.updateStatus(id, action.getAction(), requestContext);
    }

    /**
     * 修改用户密码
     *
     * @param id             用户id
     * @param passwordModel  旧密码
     * @param requestContext 请求对象
     * @return
     */
    @Operation(summary = "修改用户密码")
    @PatchMapping("/{id}/pwd")
    public Result changePwd(@PathVariable(value = Common.ID) Long id,
                            @RequestBody PasswordModel passwordModel,
                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.changePwd(id, passwordModel.getOldPassword(), passwordModel.getPassword(), requestContext);
    }

    /**
     * 重置用户密码
     *
     * @param id             用户id
     * @param passwordModel  密码
     * @param requestContext 请求对象
     * @return
     */
    @Operation(summary = "重置用户密码")
    @PatchMapping("/{id}/reset")
    public Result resetPwd(@PathVariable(value = Common.ID) Long id,
                           @RequestBody PasswordModel passwordModel,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.resetPwd(id, passwordModel.getPassword(), requestContext);
    }

    /**
     * 用户授权
     *
     * @param id             用户id
     * @param roleIds        角色id
     * @param requestContext 请求对象
     * @return
     */
    @Operation(summary = "用户授权角色")
    @PatchMapping("/{id}/accredit")
    public Result accredit(@PathVariable(value = Common.ID) Long id,
                           @RequestBody List<Long> roleIds,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.accredit(id, roleIds, requestContext);
    }

    /**
     * 检查用户是否异地登录
     *
     * @param id
     * @param request
     * @return
     */
    @Operation(summary = "检查用户是否登录")
    @GetMapping("/{id}/status")
    public Result checkUser(@PathVariable(value = Common.ID) Long id, HttpServletRequest request) {
        return userService.checkUser(id, request.getHeader(Common.TOKEN));
    }

    /**
     * 用户的角色列表
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Operation(summary = "获取角色列表")
    @GetMapping("/{id}/roles")
    public GeneralResult<List<Role>> roles(@PathVariable(value = Common.ID) Long id,
                                           @RequestParam(value = Common.CONDITION, required = false) String condition,
                                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listRoles(id, condition, requestContext);
    }

    /**
     * 获取key
     *
     * @param apiKey
     * @return
     */
    @Operation(summary = "获取key")
    @GetMapping("/security")
    public GeneralResult<AccountSecurity> secKey(@RequestParam(value = "apiKey") String apiKey) {
        return userService.getAccountSecurity(apiKey);
    }

    /**
     * 批量操作用户
     *
     * @param batchAction
     * @return
     */
    @Operation(summary = "批量操作用户")
    @PatchMapping
    public Result batchLock(@RequestBody BatchActionModel batchAction, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        switch (batchAction.getAction()) {
            case lock:
                return userService.batchLock(batchAction.getIds(), requestContext);
            case active:
                return userService.batchActive(batchAction.getIds(), requestContext);
            case remove:
                return userService.batchRemove(batchAction.getIds(), requestContext);
            default:
                return new Result(false, "不支持这种操作");
        }
    }

    /**
     * 用户移除所属租户
     *
     * @param id
     * @return
     */
    @Operation(summary = "用户移除所属租户")
    @PatchMapping("/{id}/tenants")
    public Result unBindTenants(@PathVariable Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.unBindTenants(id, requestContext);
    }

    /**
     * 登录记录查询
     *
     * @param pager
     * @return
     */
    @Operation(summary = "登录记录查询")
    @GetMapping("/track")
    public GeneralResult<GridBean<LoginRecord>> records(Pager pager) {
        return this.userService.list(pager);
    }

    @Operation(summary = "添加收藏")
    @PostMapping("/favorites/{id}")
    public Result addFavorite(@PathVariable("id") Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.addFavorite(id, requestContext);
    }

    @Operation(summary = "删除收藏")
    @DeleteMapping("/favorites/{id}")
    public Result cancelFavorite(@PathVariable("id") Long serviceId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.cancelFavorite(serviceId, requestContext);
    }

    @Operation(summary = "获取收藏夹")
    @GetMapping("/favorites")
    public GeneralResult<List<CloudServiceBean>> listFavorite(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listFavorite(requestContext);
    }

    @Operation(summary = "获取服务目录")
    @GetMapping("/services")
    public GeneralResult<List<ServiceMenuBean>> listServiceMenu(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listServiceMenu(requestContext);
    }

    @Operation(summary = "修改收藏夹排序")
    @PostMapping("/favorites/order")
    public Result changePriority(@RequestBody List<PriorityModel> priorityModels, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.changePriority(priorityModels, requestContext);
    }

    /**
     * 获取某个用户下的所有项目
     *
     * @param id
     * @return
     */
    @Operation(summary = "获取项目列表")
    @GetMapping("/{id}/projects")
    public GeneralResult<List<Project>> projects(@PathVariable(value = Common.ID) Long id) {
        return userService.projects(id);
    }

    /**
     * 导出用户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @Operation(summary = "导出项目列表")
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        Result export = userService.export(paramList, requestContext, response);
        Assert.isTrue(export.isSuccess(), "用户列表导出失败");
    }

    @Operation(summary = "下载用户列表模板")
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @RequestParam(value = "tenantId", required = false) Long tenantId,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Result result = userService.importTemp("用户列表", response);
        Assert.isTrue(result.isSuccess(), "用户模板下载失败");
    }

    @Operation(summary = "导入用户列表")
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext,
                             @RequestParam(value = "tenantId", required = false) Long tenantId, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return userService.importData(multipartFile, tenantId, requestContext);
    }

    /**
     * 锁定用户
     *
     * @param account 用户账号
     * @return 锁定结果
     */
    @Operation(summary = "锁定用户")
    @PutMapping
    @Deprecated
    public Result userLock(@RequestParam(value = Common.ACCOUNT) String account) {
        return userService.lockByAccount(account);
    }

    /**
     * 用户根据根据账号获取登录需要信息
     *
     * @param account 账号
     * @return 登录信息
     */
    @PostMapping("/info")
    @Operation(summary = "获取用户信息")
    public GeneralResult<Map<String, Object>> userAccountInfo(@RequestParam(value = Common.ACCOUNT) String account) {
        return userService.tokenLogin(account);
    }

    /**
     * 获取登录用户权限
     *
     * @param requestContext 上下文
     * @return 用户权限
     */
    @Operation(summary = "获取登录用户权限")
    @GetMapping("/permissions")
    public GeneralResult loginPermission(@Value(Common.REQ_CONTEXT) RequestContext requestContext,
                                         @RequestParam(value = "trxToken", required = false) String trxToken) {
        switch (requestContext.getCatalog()) {
            case Tenant:
                return tenantService.listPermissions(requestContext);
            case Manager:
                // 处理天融信登录用户，传递token参数 trxToken可以为null
                return managerService.listPermissions(requestContext, trxToken);
            case User:
                return userPermissionService.listPermissions(requestContext);
            default:
        }
        return new GeneralResult(false, "获取用户信息异常");
    }

    @Operation(summary = "获取登录用户按钮权限")
    @GetMapping("/button/permissions")
    public GeneralResult<List<Permission>> buttonPermissions(@Value(Common.REQ_CONTEXT) RequestContext context) {
        List<Permission> list = userPermissionService.listButtonPermissions(context);
        return new GeneralResult<>(true, list, "获取登录用户按钮权限成功");
    }
}
