package com.bocloud.sms.service;

import com.bocloud.sms.entity.Environment;
import com.bocloud.sms.interfaces.EnvironmentService;
import com.bocloud.sms.repository.EnvironmentRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Service("environmentService")
@Slf4j
public class EnvironmentServiceImpl implements EnvironmentService {

    @Autowired
    private EnvironmentRepository environmentRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<GridBean<Environment>> list(Pager pager, RequestContext context) {
        GridBean gridBean = null;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            sorter.put("gmtCreate", Common.ONE);
            Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
            Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
            int total = this.environmentRepository.count(params);
            if (simple) {
                List<SimpleBean> beans = this.environmentRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<Environment> beans = this.environmentRepository.list(page, rows, params, sorter);
                gridBean = GridHelper.getBean(page, rows, total, beans);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query Environment list fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }

    }

    @Override
    public GeneralResult<Environment> create(Environment environment, RequestContext context) {
        try {
            if (environmentRepository.existName(environment.getName())) {
                return new GeneralResult<Environment>(false, "名称已存在");
            }
            environment.setCreatorId(context.getTarget());
            environment.setMenderId(context.getTarget());
            this.environmentRepository.save(environment);
            return new GeneralResult<Environment>(true, "添加成功");
        } catch (Exception e) {
            log.error("creat environment error:", e);
            return new GeneralResult<Environment>(false, "添加失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, RequestContext context) {
        String path = Environment.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Environment environment = this.environmentRepository.query(id);
            if (null == environment) {
                log.warn("Environment data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            boolean result = this.environmentRepository.remove(id, context.getTarget());
            if (result) {
                return new GeneralResult<>(true, "删除成功");
            } else {
                return new GeneralResult<>(false, "删除失败");
            }
        } catch (Exception e) {
            log.error("remove Environment fail:", e);
            return new GeneralResult<>(false, "删除环境失败");
        }
    }

    @Override
    public GeneralResult<Environment> detail(Long id) {
        try {
            Environment environment = this.environmentRepository.query(id);
            if (null == environment) {
                log.warn("Environment data not exist");
                return new GeneralResult<Environment>(false, "数据不存在");
            }
            return new GeneralResult<Environment>(true, environment, "查询详情成功");
        } catch (Exception e) {
            log.error("query Environment detail fail:", e);
            return new GeneralResult<Environment>(false, "查询详情失败");
        }
    }

    @Override
    public GeneralResult<Environment> modify(Environment bean, RequestContext context) {
        String path = Environment.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Environment environment = this.environmentRepository.query(bean.getId());
            if (null == environment) {
                log.warn("Environment data not exist");
                return new GeneralResult<Environment>(false, "数据不存在");
            }
            if (!environment.getName().equals(bean.getName())) {
                if (environmentRepository.existName(bean.getName())) {
                    log.warn("Environment already exist!");
                    return new GeneralResult<Environment>(false, "环境名称已存在");
                }
            }
            environment.setName(bean.getName());
            environment.setRemark(bean.getRemark());
            environment.setMenderId(context.getTarget());
            environment.setGmtModify(new Date());
            this.environmentRepository.update(environment);
            return new GeneralResult<Environment>(true, environment, "修改成功");
        } catch (Exception e) {
            log.error("modify Environment fail:", e);
            return new GeneralResult<Environment>(false, "修改环境失败");
        }
    }

}
