package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.entity.Plugin;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.interfaces.PluginService;
import com.bocloud.sms.model.PluginInfoModel;
import com.bocloud.sms.repository.PermissionRepository;
import com.bocloud.sms.repository.PluginRepository;
import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.core.descriptor.PluginDescriptor;
import com.gitee.starblues.integration.operator.EmptyPluginOperator;
import com.gitee.starblues.integration.operator.PluginOperator;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.client.discovery.DiscoveryClient;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * 插件
 *
 * @author zhangyf
 * @since 2022-05-19
 */
@RequiredArgsConstructor
@Slf4j
@Service
public class PluginServiceImpl implements PluginService {

    private final PluginRepository pluginRepository;
    private final LockFactory lockFactory;
    private final CurrentService currentService;
    private final DictService dictService;
    private final PermissionRepository permissionRepository;
    private final StringRedisTemplate stringRedisTemplate;
    private final DiscoveryClient discoveryClient;
    private PluginOperator pluginOperator;
    @Value("${plugin.repository.path:/home/cmp/plugins/repository/}")
    private String pluginRepoPath;

    @PostConstruct
    protected void init() {
        this.pluginOperator = new EmptyPluginOperator();
    }

    @Override
    public GeneralResult<GridBean<Plugin>> list(Pager pager) {
        pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = this.pluginRepository.count(pager.getParams());
        List<Plugin> plugin = this.pluginRepository.list(pager);
        GridBean<Plugin> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, plugin);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @Override
    public Map<String, List<Plugin>> listInstalled(String service) {
        List<String> instances = discoveryClient.getInstances(service).stream().map(instance -> instance.getHost() + ":" + instance.getPort())
                .collect(Collectors.toList());
        List<Plugin> plugins = this.runningPlugins(service);
        List<Plugin> result = new ArrayList<>();
        Set<String> installNames = new HashSet<>();
        instances.forEach(instance -> {
            String key = service + "_plugins_" + instance;
            Map<Object, Object> pluginMap = stringRedisTemplate.opsForHash().entries(key);
            plugins.forEach(plugin -> {
                        Plugin clonePlugin = JSON.parseObject(JSON.toJSONString(plugin), Plugin.class);
                        if (null != pluginMap.get(plugin.getCode()) && !"null".equals(pluginMap.get(plugin.getCode()))) {
                            PluginInfoModel pluginInfoModel = JSON.parseObject(pluginMap.get(plugin.getCode()).toString(), PluginInfoModel.class);
                            if (pluginInfoModel.getPluginDescriptor().getPluginVersion().equals(plugin.getVersion())) {
                                clonePlugin.setPluginState(pluginInfoModel.getPluginState());
                                clonePlugin.setAddress(instance);
                                result.add(clonePlugin);
                                installNames.add(clonePlugin.getName());
                            }
                        } else {
                            clonePlugin.setPluginState(null);
                            clonePlugin.setAddress(instance);
                            result.add(clonePlugin);
                        }
                    }
            );
        });
        Map<String, List<Plugin>> map = new HashMap<>();
        result.stream().collect(Collectors.groupingBy(Plugin::getName)).forEach((k, v) -> {
            if (installNames.contains(k)) {
                List<Plugin> distinctPlugins = new ArrayList<>();
                Set<String> set = new HashSet<>();
                v.forEach(plugin -> {
                    if (!set.contains(plugin.getName() + plugin.getAddress())) {
                        distinctPlugins.add(plugin);
                        set.add(plugin.getName() + plugin.getAddress());
                    }
                });
                map.put(k, distinctPlugins);
            }
        });
        return map;
    }

    @Override
    public Map<String, List<Plugin>> listUpdate(String service) {
        List<String> instances = discoveryClient.getInstances(service).stream().map(instance -> instance.getHost() + ":" + instance.getPort())
                .collect(Collectors.toList());
        List<Plugin> plugins = this.runningPlugins(service);
        List<Plugin> result = new ArrayList<>();
        Set<String> canUpdateNames = new HashSet<>();
        Map<String, List<Plugin>> all = plugins.stream().collect(Collectors.groupingBy(Plugin::getName));
        instances.forEach(instance -> {
            String key = service + "_plugins_" + instance;
            Map<Object, Object> pluginMap = stringRedisTemplate.opsForHash().entries(key);
            plugins.forEach(plugin -> {
                        Plugin clonePlugin = JSON.parseObject(JSON.toJSONString(plugin), Plugin.class);
                        if (null != pluginMap.get(plugin.getCode()) && !"null".equals(pluginMap.get(plugin.getCode()))) {
                            PluginInfoModel pluginInfoModel = JSON.parseObject(pluginMap.get(plugin.getCode()).toString(), PluginInfoModel.class);
                            if (pluginInfoModel.getPluginDescriptor().getPluginVersion().equals(plugin.getVersion())) {
                                clonePlugin.setPluginState(pluginInfoModel.getPluginState());
                                clonePlugin.setAddress(instance);
                                boolean canUpdate = all.get(clonePlugin.getName()).stream()
                                        .anyMatch(p -> canUpdate(pluginInfoModel.getPluginDescriptor().getPluginVersion(), p.getVersion()));
                                clonePlugin.setCanUpdate(canUpdate);
                                if (clonePlugin.getCanUpdate()) {
                                    canUpdateNames.add(clonePlugin.getName());
                                }
                                result.add(clonePlugin);
                            }
                        } else {
                            clonePlugin.setPluginState(null);
                            clonePlugin.setAddress(instance);
                            result.add(clonePlugin);
                        }
                    }
            );
        });
        Map<String, List<Plugin>> map = new HashMap<>();
        result.stream().collect(Collectors.groupingBy(Plugin::getName)).forEach((k, v) -> {
            if (canUpdateNames.contains(k)) {
                List<Plugin> distinctPlugins = new ArrayList<>();
                Set<String> set = new HashSet<>();
                v.forEach(plugin -> {
                    if (!set.contains(plugin.getName() + plugin.getAddress())) {
                        distinctPlugins.add(plugin);
                        set.add(plugin.getName() + plugin.getAddress());
                    }
                });
                map.put(k, distinctPlugins);
            }
        });
        return map;
    }

    @Override
    public Map<String, List<Plugin>> listCanInstall(String service) {
        List<String> instances = discoveryClient.getInstances(service).stream().map(instance -> instance.getHost() + ":" + instance.getPort())
                .collect(Collectors.toList());
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("service", service), Sign.EQ));
        List<Plugin> plugins = this.pluginRepository.list(new Pager(1, Integer.MAX_VALUE, params, null));
        List<Plugin> result = new ArrayList<>();
        Set<String> canInstallNames = new HashSet<>();
        instances.forEach(instance -> {
            String key = service + "_plugins_" + instance;
            Map<Object, Object> pluginMap = stringRedisTemplate.opsForHash().entries(key);
            plugins.forEach(plugin -> {
                        Plugin clonePlugin = JSON.parseObject(JSON.toJSONString(plugin), Plugin.class);
                        if (null != pluginMap.get(plugin.getCode()) && !"null".equals(pluginMap.get(plugin.getCode()))) {
                            PluginInfoModel pluginInfoModel = JSON.parseObject(pluginMap.get(plugin.getCode()).toString(), PluginInfoModel.class);
                            if (pluginInfoModel.getPluginDescriptor().getPluginVersion().equals(plugin.getVersion())) {
                                clonePlugin.setPluginState(pluginInfoModel.getPluginState());
                                clonePlugin.setAddress(instance);
                                result.add(clonePlugin);
                            }
                        } else {
                            clonePlugin.setPluginState(null);
                            clonePlugin.setAddress(instance);
                            result.add(clonePlugin);
                            canInstallNames.add(clonePlugin.getName());
                        }
                    }
            );
        });
        Map<String, List<Plugin>> map = new HashMap<>();
        result.stream().collect(Collectors.groupingBy(Plugin::getName)).forEach((k, v) -> {
            if (canInstallNames.contains(k)) {
                List<Plugin> distinctPlugins = new ArrayList<>();
                Set<String> set = new HashSet<>();
                v.forEach(plugin -> {
                    if (!set.contains(plugin.getName() + plugin.getAddress())) {
                        distinctPlugins.add(plugin);
                        set.add(plugin.getName() + plugin.getAddress());
                    }
                });
                map.put(k, distinctPlugins);
            }
        });
        return map;
    }


    /**
     * 创建plugin
     * 创建插件默认都是开启状态
     *
     * @param plugin 插件对象
     * @return 成功/失败
     */
    @Override
    public Result create(Plugin plugin) {
        try (AutoCloseLock acLock = lockFactory.getACLock(Plugin.class.getSimpleName())) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            File destFile = new File(pluginRepoPath + File.separator + plugin.getService(), plugin.getPath());
            Path tempFilePath = Paths.get(destFile.getPath());
            PluginInfo pluginInfo = pluginOperator.parse(tempFilePath);

            PluginDescriptor pluginDescriptor = pluginInfo.getPluginDescriptor();
            plugin.setRemark(pluginDescriptor.getDescription());
            plugin.setCode(pluginDescriptor.getPluginId());
            plugin.setVersion(pluginInfo.getPluginDescriptor().getPluginVersion());
            plugin.setProvider(pluginDescriptor.getProvider());

            Plugin dbPlugin = this.pluginRepository.getByCode(plugin.getCode(), plugin.getVersion());
            Plugin dbPluginByName = this.pluginRepository.getByName(plugin.getName(), plugin.getVersion());
            Assert.isNull(dbPluginByName, "插件名称已存在！");
            Assert.isNull(dbPlugin, "插件编码已存在！");
            // 默认插件状态就是开启
            plugin.setStatus(true);
            this.pluginRepository.save(plugin);
            return new Result(true, "新增成功");
        }
    }

    @Override
    @Transactional
    public Result modify(Long id, Plugin plugin, boolean hasPackage) {
        try (AutoCloseLock acLock = lockFactory.getACLock(Plugin.class.getSimpleName())) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Plugin dbPlugin = this.pluginRepository.get(Plugin.class, id);
            Assert.notNull(dbPlugin, "未查询到任何数据!");

            if (!plugin.getName().equals(dbPlugin.getName())) {
                boolean exists = pluginRepository.existsSameName(plugin.getName());
                Assert.isTrue(!exists, "插件名称已存在");
            }
            dbPlugin.setName(plugin.getName());
            dbPlugin.setVendorType(plugin.getVendorType());

            if (hasPackage) {
                File destFile = new File(pluginRepoPath + File.separator + plugin.getService(), plugin.getPath());
                Path tempFilePath = Paths.get(destFile.getPath());
                PluginInfo pluginInfo = pluginOperator.parse(tempFilePath);
                if (!pluginInfo.getPluginId().equals(dbPlugin.getCode())) {
                    boolean exists = pluginRepository.existsSameCode(plugin.getCode());
                    Assert.isTrue(!exists, "插件编码已存在");
                }
                PluginDescriptor pluginDescriptor = pluginInfo.getPluginDescriptor();
                dbPlugin.setRemark(pluginDescriptor.getDescription());
                dbPlugin.setCode(pluginDescriptor.getPluginId());
                dbPlugin.setVersion(pluginDescriptor.getPluginVersion());
                dbPlugin.setProvider(pluginDescriptor.getProvider());
                dbPlugin.setSize(plugin.getSize());
                dbPlugin.setPath(plugin.getPath());
                dbPlugin.setService(plugin.getService());
                dbPlugin.setName(plugin.getName());
                dbPlugin.setVendorType(plugin.getVendorType());

            }
            this.pluginRepository.update(dbPlugin);
            // 修改时需要将权限表绑定的内容同步修改
            permissionRepository.updateByPluginsCode(plugin.getCode(), dbPlugin.getCode());
            return new Result(true, "插件修改成功");
        }
    }

    /**
     * 删除plugin
     *
     * @param id 插件id
     * @return 成功/失败
     */
    @Override
    @Transactional
    public Result remove(Long id) {
        try (AutoCloseLock acLock = lockFactory.getACLock(Plugin.class.getSimpleName())) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Plugin plugin = this.pluginRepository.get(Plugin.class, id);
            Assert.notNull(plugin, " 数据不存在");
            plugin.setDeleted(true);
            this.pluginRepository.update(plugin);
            // 删除插件时，解绑权限表中所有的菜单
            // 需要将权限表中的插件名称改为""  并将权限表中的插件状态修改为false
            this.permissionRepository.unBindingPlugins(plugin.getCode(), false, null);
            return new Result(true, "插件删除成功!");
        }
    }

    /**
     * 批量删除
     *
     * @param idList 删除插件列表集合
     * @return 成功/失败 个数
     */
    @Override
    @Transactional
    public Result batchRemove(List<Long> idList) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(idList, " 至少选择一个插件删除！");
            for (Long id : idList) {
                this.remove(id);
                rightCount++;
            }
        } catch (Exception e) {
            log.error("plugin batch delete error!", e);
            return new Result(false, "批量删除插件失败:" + e.getMessage());
        }
        return new Result(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }

    /**
     * 查询单条plugin
     *
     * @param id 插件id
     * @return 插件详细信息
     */
    @Override
    public Plugin detail(Long id) {
        Plugin plugin = this.pluginRepository.get(Plugin.class, id);
        Assert.notNull(plugin, "未查询到任何数据!");
        Assert.isTrue(!plugin.getDeleted(), "查询失败,数据不存在");
        return plugin;
    }

    /**
     * 获取平台列表(资源接入)
     *
     * @param pager 分页参数
     * @return 资源纳管时展示的云平台列表
     */
    @Override
    public GeneralResult<List<Dictionary>> getVendorList(Pager pager) {
        List<Plugin> pluginList = this.runningPlugins(BoCloudService.CMP.getName());
        GeneralResult<List<Dictionary>> resourceType = dictService.childTree("RESOURCE_TYPE");
        List<String> resourceTypeList = new ArrayList<>();
        Optional.ofNullable(pluginList).orElse(new ArrayList<>()).forEach(p -> resourceTypeList.add(p.getVendorType()));
        List<Dictionary> dictionaries = resourceType.getData();
        if (!ListTool.isEmpty(dictionaries)) {
            for (int i = dictionaries.size() - 1; i >= 0; i--) {
                Dictionary dictionary = dictionaries.get(i);
                List<Dictionary> children = dictionary.getChildren();
                for (int j = children.size() - 1; j >= 0; j--) {
                    Dictionary childrenDictionary = children.get(j);
                    if (!resourceTypeList.contains(childrenDictionary.getValue())) {
                        children.remove(childrenDictionary);
                    }
                }
                if (ListTool.isEmpty(children)) {
                    dictionaries.remove(dictionary);
                }
            }
        }
        // 获取到插件列表中的资源类型
        return new GeneralResult<>(true, dictionaries, "查询成功");
    }

    @Override
    public List<Plugin> listLicense(Pager pager) {
        List<String> plugins = LicenseDaemon.instance(currentService).content().getPlugins();
        List<Plugin> pluginList = new ArrayList<>();
        Optional.ofNullable(plugins).orElse(new ArrayList<>()).forEach(p -> pluginList.add(new Plugin(p, false)));
        List<Plugin> dbPluginList = pluginRepository.list(new Pager(1, Integer.MAX_VALUE, null, null));
        log.debug("获取到License中的插件列表为： " + pluginList);
        log.debug("获取到数据库中的插件列表为： " + dbPluginList);
        Optional.ofNullable(dbPluginList).orElse(new ArrayList<>()).forEach(dbPlugin -> pluginList.forEach(p -> {
            if (p.getCode().equals(dbPlugin.getCode())) {
                p.setId(dbPlugin.getId() != null ? dbPlugin.getId() : null);
                p.setStatus(dbPlugin.getStatus());
                p.setName(dbPlugin.getName());
                p.setRemark(dbPlugin.getRemark());
                p.setVendorType(dbPlugin.getVendorType());
            }
        }));
        return pluginList;
    }

    @Override
    public List<Plugin> runningPlugins(String... services) {
        Set<Object> pluginCodes = new HashSet<>();
        for (String service : services) {
            List<String> instances = discoveryClient.getInstances(service).stream().map(instance -> instance.getHost() + ":" + instance.getPort())
                    .collect(Collectors.toList());
            instances.forEach(instance -> {
                String key = service + "_plugins_" + instance;
                Map<Object, Object> pluginMap = stringRedisTemplate.opsForHash().entries(key);
                pluginCodes.addAll(pluginMap.keySet());
            });
        }
        return this.pluginRepository.listByCode(pluginCodes.stream().map(code -> (String) code).collect(Collectors.toList()));
    }

    /**
     * 校验版本号是否可更新
     */
    private boolean canUpdate(String currVersion, String newVersion) {
        if (currVersion.equals(newVersion)) {
            return false;
        }
        if (StringUtils.isBlank(currVersion)) {
            return false;
        }
        if (StringUtils.isBlank(newVersion)) {
            return false;
        }
        String[] currSubVersions = currVersion.split("\\.");
        String[] newSubVersions = newVersion.split("\\.");
        if (currSubVersions.length < 3 && newSubVersions.length < 3) {
            return false;
        }
        if (currSubVersions[0].equals(newSubVersions[0]) && currSubVersions[1].equals(newSubVersions[1])) {
            long c = Long.parseLong(currSubVersions[2]);
            long n = Long.parseLong(newSubVersions[2]);
            return n > c;
        }
        return false;
    }
}
