package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.interfaces.SystemConfigService;
import com.bocloud.sms.model.ConfigModel;
import com.bocloud.sms.repository.AccountSecurityRepository;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.ldap.LdapConnector;
import com.bocloud.sms.service.ldap.LdapRepository;
import com.bocloud.sms.service.utils.*;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 系统设置Service层
 *
 * @author lyy
 * @Version 1.0
 * @since 2017年8月18日
 */
@RequiredArgsConstructor
@Service("systemConfigService")
@Slf4j
public class SystemConfigServiceImpl implements SystemConfigService {

    @Value("${spring.upload.path:/home/cmp}")
    private String storagePath;
    private final LockFactory lockFactory;
    private final SystemConfigRepository systemConfigRepository;
    private final ConfigRegister configRegister;
    private final UserRepository userRepository;
    private final AccountSecurityRepository accountSecurityRepository;
    private final AdmissionUtil admissionUtil;
    private final DepartmentRepository departmentRepository;
    private final EmailSender emailSender;
    private final WeChatWorkSender weChatWorkSender;
    private final DingTalkSender dingTalkSender;

    /**
     * 修改系统配置
     *
     * @param systemConfig 系统配置对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(SystemConfig systemConfig) throws InternalServerException {
        String path = SystemConfig.class.getSimpleName() + "_" + systemConfig.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemConfig object = this.systemConfigRepository.query(systemConfig.getId());
            Assert.notNull(object, "SystemConfig does not exist!");
            BeanUtils.copyProperties(systemConfig, object);
            this.systemConfigRepository.modify(object);
            object.setGmtModify(new Date());
            // 注册zk
            List<SystemConfig> configs = systemConfigRepository.listAll();
            configRegister.updateConfig(configs);
            return new Result(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify SysTemConfig fail:", e);
            throw new InternalServerException("修改失败," + e.getMessage(), e, e.getMessage());
        }
    }


    /**
     * 批量更新系统配置
     *
     * @param models 系统配置集合
     * @return
     */
    @Override
    public Result batchUpdate(List<ConfigModel> models, Map<String, MultipartFile> fileMap, RequestContext requestContext) {
        String[] titles = new String[]{"id", "value", "gmtModify", "menderId"};
        try {
            for (Map.Entry<String, MultipartFile> entry : fileMap.entrySet()) {
                String code = entry.getKey();
                MultipartFile multipartFile = entry.getValue();
                String originalFilename = multipartFile.getOriginalFilename();
                Assert.notNull(originalFilename, "文件名不能为空");
                String fileName = System.currentTimeMillis() + originalFilename.substring(originalFilename.lastIndexOf("."));
                String filePath = storagePath + "/config-files/";
                File file = new File(filePath, fileName);
                if (!file.exists()) {
                    file.mkdirs();
                }
                multipartFile.transferTo(file);
                String realPath = "/config-files/" + fileName;
                models.add(new ConfigModel(code, realPath));
            }
            Map<String, String> configMap = new HashMap<>(8);
            // 前端传过来需要修改的code
            Set<String> codeSet = models.stream().map(ConfigModel::getCode).collect(Collectors.toSet());

            // 前端传过来的 map <code,value>
            Map<String, String> modelsCodeValue = models.stream()
                    .filter(configModel -> configModel.getValue() != null)
                    .collect(Collectors.toMap(ConfigModel::getCode, ConfigModel::getValue));

            // 数据库中存储的需要修改的 SystemConfigList
            List<SystemConfig> systemConfigs = systemConfigRepository.listByCodes(codeSet);
            // 筛选出数据库中需要修改的敏感字段的code
            List<String> sensitiveInformationCodes = systemConfigs.stream()
                    .filter(Objects::nonNull)
                    .filter(systemConfig -> "PASSWORD".equals(systemConfig.getType()))
                    .map(SystemConfig::getCode).collect(Collectors.toList());
            // 筛选出数据库中需要修改的敏感字段的 map<code,value>
            Map<String, String> sensitiveInformation = systemConfigs.stream()
                    .filter(Objects::nonNull)
                    .filter(systemConfig -> "PASSWORD".equals(systemConfig.getType()))
                    .collect(Collectors.toMap(s -> Optional.ofNullable(s.getCode()).orElse("空的"), s -> Optional.ofNullable(s.getValue()).orElse("空的"), (a, b) -> b, HashMap::new));

            for (String sensitiveInformationCode : sensitiveInformationCodes) {
                if (null != modelsCodeValue.get(sensitiveInformationCode) && null != sensitiveInformation.get(sensitiveInformationCode)) {
                    // 如果前端传过来的数据和数据库中存储的数据一致，不需要加密，防止二次加密
                    boolean equals = modelsCodeValue.get(sensitiveInformationCode).equals(sensitiveInformation.get(sensitiveInformationCode));
                    if (equals) {
                        configMap.put(sensitiveInformationCode, modelsCodeValue.get(sensitiveInformationCode));
                    } else {
                        configMap.put(sensitiveInformationCode, new DESEncryptor().encrypt(modelsCodeValue.get(sensitiveInformationCode), "BocloudCMP"));
                    }
                }
            }
            // 处理非敏感字段
            for (Map.Entry<String, String> stringStringEntry : modelsCodeValue.entrySet()) {
                if (!configMap.containsKey(stringStringEntry.getKey())) {
                    configMap.put(stringStringEntry.getKey(), stringStringEntry.getValue());
                }
            }
            Set<String> configCodes = configMap.keySet();
            if (!configCodes.isEmpty()) {
                List<SystemConfig> list = systemConfigRepository.listByCodes(configCodes);
                for (SystemConfig systemConfig : list) {
                    systemConfig.setValue(configMap.get(systemConfig.getCode()));
                    systemConfig.setMenderId(requestContext.getTarget());
                    systemConfig.setGmtModify(new Date());
                }
                boolean flag = systemConfigRepository.batchUpdate(list, titles);
                if (!flag) {
                    return new GeneralResult<>(false, "修改失败");
                }
            }
            // 注册zk
            List<SystemConfig> configs = systemConfigRepository.listAll();
            configRegister.updateConfig(configs);
            //访问控制如果修改了 则刷新缓存
            if (configCodes.contains("accessStrategy")) {
                admissionUtil.refresh();
            }
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("batch modify  system config  fail:", e);
            return new GeneralResult<>(false, "修改失败" + e.getMessage());
        }
    }

    @Override
    public Result syncLdap() {
        try {
            SystemConfig urlConfig = systemConfigRepository.queryByCode("LdapUrl");
            if (null == urlConfig || !StringUtils.hasText(urlConfig.getValue())) {
                return new Result(false, "Ldap地址配置为空");
            }
            SystemConfig userConfig = systemConfigRepository.queryByCode("LdapUsername");
            if (null == userConfig || !StringUtils.hasText(userConfig.getValue())) {
                return new Result(false, "Ldap用户配置为空");
            }
            SystemConfig pswdConfig = systemConfigRepository.queryByCode("LdapPassword");
            if (null == pswdConfig || !StringUtils.hasText(pswdConfig.getValue())) {
                return new Result(false, "Ldap密码配置为空");
            }
            SystemConfig baseConfig = systemConfigRepository.queryByCode("LdapBaseDn");
            if (null == baseConfig || !StringUtils.hasText(baseConfig.getValue())) {
                return new Result(false, "Ldap基本路径配置为空");
            }
            LdapConfig config = new LdapConfig(urlConfig.getValue(), userConfig.getValue(), new DESEncryptor().decrypt(pswdConfig.getValue(), "BocloudCMP"),
                    baseConfig.getValue());
            LdapTransportor transportor = new LdapTransportor(userRepository, departmentRepository, accountSecurityRepository, config);
            if (!transportor.accessable()) {
                return new Result(false, "Ldap平台认证失败");
            }
            ThreadPoolExecutor ldapExecutor = new ThreadPoolExecutor(1, 1, 5, TimeUnit.MINUTES, new ArrayBlockingQueue<>(1), new ThreadFactoryBuilder().setNameFormat("ldapExecutor").build());
            ldapExecutor.execute(() -> {
                try (AutoCloseLock lock = lockFactory.getACLock("ldapLock")) {
                    if (!lock.acquire(10, TimeUnit.SECONDS)) {
                        log.warn("正在进行Ldap同步任务……");
                    } else {
                        transportor.sync();
                    }
                } catch (Exception e) {
                    log.error("running ldap task error:{}", e.getMessage());
                }
            });
            ldapExecutor.shutdown();
            return new Result(true, "Ldap同步任务已下发");
        } catch (Exception e) {
            log.error("sync ldap data error:", e);
            return new Result(false, "同步失败");
        }
    }

    @Override
    public GeneralResult<Map<String, String>> listConfigs(String category, List<String> codes) {
        try {
            List<SystemConfig> systemConfigs;
            if (StringUtils.hasText(category)) {
                systemConfigs = systemConfigRepository.listByCategory(category);
            } else {
                systemConfigs = systemConfigRepository.listByCodes(new HashSet<>(codes));
            }
            Map<String, SystemConfig> configMap = systemConfigs.stream().collect(Collectors.toMap(SystemConfig::getCode, Function.identity()));
            Map<String, String> result = new HashMap<>(8);
            for (Map.Entry<String, SystemConfig> entry : configMap.entrySet()) {
                String value = entry.getValue().getValue();
                result.put(entry.getKey(), value);
            }
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("list configs by codes error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<JSONArray> listCategoryTree(String category) {
        try {
            List<SystemConfig> systemConfigs = systemConfigRepository.listByCategory(category);
            Map<String, List<SystemConfig>> configMap = systemConfigs.stream().collect(Collectors.groupingBy(SystemConfig::getSubclass));
            JSONArray jsonArray = new JSONArray();
            for (Map.Entry<String, List<SystemConfig>> entry : configMap.entrySet()) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", entry.getKey());
                jsonObject.put("values", entry.getValue());
                jsonArray.add(jsonObject);
            }
            return new GeneralResult<>(true, jsonArray, "查询成功");
        } catch (Exception e) {
            log.error("list configs tree by category error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result configTest(String category) {
        try {
            switch (category) {
                case "email":
                    if (emailSender.testConnected()) {
                        return new Result(true, "连接测试成功");
                    }
                    break;
                case "dingtalk":
                    String accessToken = dingTalkSender.getAccessToken(true);
                    if (StringUtils.hasText(accessToken)) {
                        return new Result(true, "连接测试成功");
                    }
                    break;
                case "wechat":
                    JSONObject jsonObject = JSONObject.parseObject(weChatWorkSender.getToken());
                    int errCode = jsonObject.getInteger("errcode");
                    if (errCode == 0) {
                        accessToken = jsonObject.getString("access_token");
                        if (StringUtils.hasText(accessToken)) {
                            return new Result(true, "连接测试成功");
                        }
                    }
                    break;
                case "ldap":
                    SystemConfig urlConfig = systemConfigRepository.queryByCode("LdapUrl");
                    String url = urlConfig.getValue();
                    if (null == urlConfig || !StringUtils.hasText(url)) {
                        return new Result(false, "Ldap地址配置为空");
                    }
                    SystemConfig userConfig = systemConfigRepository.queryByCode("LdapUsername");
                    String userName = userConfig.getValue();
                    if (null == userConfig || !StringUtils.hasText(userName)) {
                        return new Result(false, "Ldap用户配置为空");
                    }
                    SystemConfig pswdConfig = systemConfigRepository.queryByCode("LdapPassword");
                    String pswd = new DESEncryptor().decrypt(pswdConfig.getValue(), "BocloudCMP");
                    if (null == pswdConfig || !StringUtils.hasText(pswd)) {
                        return new Result(false, "Ldap密码配置为空");
                    }
                    SystemConfig baseConfig = systemConfigRepository.queryByCode("LdapBaseDn");
                    String baseDn = baseConfig.getValue();
                    if (null == baseConfig || !StringUtils.hasText(baseDn)) {
                        return new Result(false, "Ldap基本路径配置为空");
                    }

                    LdapConnector ldapConnector = new LdapConnector(url, userName, pswd, baseDn);
                    LdapRepository ldapRepository = new LdapRepository(ldapConnector);
                    ldapConnector.start();
                    if (ldapConnector.accessable() && !StringUtils.isEmpty(ldapConnector.accessable())) {
                        return new Result(true, "连接测试成功");
                    }
                    break;
                default:
                    return Result.FAILED("不支持这种操作");
            }
        } catch (Exception e) {
            log.error("test {} connection error,", category, e);
        }
        return Result.FAILED("测试连接失败");
    }

    @Override
    public GeneralResult<SystemConfig> queryByCode(String code) {
        String errorMsg;

        try {
            SystemConfig config = systemConfigRepository.queryByCode(code);
            if (config == null) {
                return new GeneralResult<>(false, "未查询到code为" + code + "的配置信息");
            }
            String value = config.getValue();
            if (value != null) {
                config.setValue(value.trim());
            }
            return new GeneralResult<>(true, config, "查询成功");
        } catch (Exception e) {
            errorMsg = "通过code查询系统配置信息出错，" + e.getMessage();
            log.error(errorMsg, e);
            return new GeneralResult<>(false, errorMsg);
        }
    }
}
