package com.bocloud.sms.service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudStatus;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.enums.QuotaTargetCatalog;
import com.bocloud.sms.interfaces.PluginService;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.model.*;
import com.bocloud.sms.repository.*;
import com.bocloud.sms.service.internel.CmpInternalService;
import com.bocloud.sms.service.internel.CosInternelService;
import com.bocloud.sms.service.internel.CosQuotaService;
import com.bocloud.sms.service.utils.ExportUtil;
import com.bocloud.sms.service.utils.ImportExcelUtil;
import com.bocloud.sms.utils.FavoriteComparator;
import com.bocloud.sms.utils.PermissionComparator;
import com.bocloud.sms.utils.PermitComparator;
import com.google.common.collect.Lists;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.encrypt.SHAEncryptor;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.*;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.common.utils.Tokens;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import io.jsonwebtoken.SignatureAlgorithm;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.bocloud.sms.service.utils.ExportUtil.*;

/**
 * 租户抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.05
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class TenantServiceImpl implements TenantService {

    private static final String TENANT_GRANT_QUEUE = "tenant.grant.queue";
    private final CurrentService currentService;
    private final StringRedisTemplate redisTemplate;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final RoleRepository roleRepository;
    private final PermissionRepository permissionRepository;
    private final DepartmentRepository departmentRepository;
    private final AccountSecurityRepository securityRepository;
    private final ProjectRepository projectRepository;
    private final BusinessRepository businessRepository;
    private final LockFactory lockFactory;
    private final CosQuotaService cosQuotaService;
    private final CmpInternalService cmpInternalService;
    private final CosInternelService cosInternelService;
    private final AmqpTemplate amqpTemplate;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final TenantPoolRepository tenantPoolRepository;
    private final PoolGroupRepository poolGroupRepository;
    private final CloudServiceBeanRepository cloudServiceBeanRepository;
    private final FavoriteRepository favoriteRepository;
    private final LoginRecordRepository loginRecordRepository;
    private final ApiPermissionRepository apiPermissionRepository;
    private final PluginService pluginService;
    /**
     * 菜单权限是否使用插件校验
     */
    @Value("${plugin.permission.enable:true}")
    private boolean validatePlugin;

    /**
     * 新建租户
     *
     * @param tenantModel
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result create(TenantModel tenantModel, RequestContext requestContext) throws InternalServerException {
        Tenant tenant = new Tenant();
        BeanUtils.copyProperties(tenantModel, tenant);
        Assert.notNull(tenant, "tenant can't be null");
        tenant.setCreatorId(requestContext.getTarget());
        // 初始化密码
        String salt = UUID.randomUUID().toString();
        Encryptor encryptor = new SHAEncryptor();
        try {
            // 创建保存租户基本信息
            Tenant oldTenant = tenantRepository.getByEmail(tenant.getEmail());
            Assert.isNull(oldTenant, "该邮箱已被注册");
            Tenant existAccount = tenantRepository.getByAccount(tenant.getAccount());
            Assert.isNull(existAccount, "该账号已存在");
            Tenant existName = tenantRepository.getByName(tenant.getName());
            Assert.isNull(existName, "该租户已存在");
            Assert.isNull(tenantRepository.getByCode(tenant.getGroupCode()), "该租户编码已存在");
            String password = encryptor.encrypt(new AESEncryptor().decrypt(tenant.getPassword().trim(), null), salt);
            tenant.setPassword(password);
            // 关键信息去空
            tenant.setAccount(tenant.getAccount().trim());
            tenant.setName(tenant.getName().trim());
            tenant.setStatus(BaseStatus.NORMAL.name());
            tenant.setLoginStatus(false);
            tenant.setLastPwdModifyDate(new Date());
            tenantRepository.save(tenant);
            String account = tenant.getAccount();
            Tenant cacheTenant = tenantRepository.getByAccount(account);
            // 添加租户信息缓存
            redisTemplate.opsForValue().set(Catalog.Tenant.name() + "_" + cacheTenant.getId(), account);
            redisTemplate.opsForValue().set(Catalog.Tenant.name() + "_Name_" + cacheTenant.getId(),
                    cacheTenant.getName());
            // 构建租户随机数对象
            AccountSecurity security = new AccountSecurity();
            String data = Long.toString(System.currentTimeMillis());
            String apiKey;
            String secKey;
            try {
                apiKey = encryptor.encrypt(data, Common.API_KEY);
                secKey = encryptor.encrypt(data, Common.SEC_KEY);
            } catch (Exception e) {
                log.error("Get error message:", e);
                return new Result(false, "用户创建失败");
            }
            security.setTargetId(tenant.getId());
            security.setCategory(AccountSecurity.Category.Tenant);
            security.setSalt(salt);
            security.setApiKey(apiKey);
            security.setSecKey(secKey);
            security.setTenantId(tenant.getId());
            boolean success = securityRepository.baseSave(security);
            if (!success) {
                this.callback(tenant.getId(), requestContext);
                return new Result(false, "租户创建失败");
            }
        } catch (Exception e) {
            log.error("create tenant fail:", e);
            throw new InternalServerException("添加租户失败," + e.getMessage(), e, e.getMessage());
        }
        try {
            // 创建租户授信账号对象
            TenantAccount tenantAccount = new TenantAccount(tenant.getId(), BigDecimal.ZERO, BigDecimal.ZERO,
                    BigDecimal.ZERO, BigDecimal.ZERO, true);
            // 获取授权服务
            List<Long> serviceIds = new ArrayList<>();
            if (ListTool.hasElement(tenantModel.getServices())) {
                serviceIds = tenantModel.getServices().stream().map(ServiceBean::getServiceId).collect(Collectors.toList());
            }
            TenantGrantMessage tenantGrantMessage =
                    new TenantGrantMessage(tenant.getId(), serviceIds, tenantAccount, requestContext);
            // 添加收藏夹
            List<Favorite> favorites = new ArrayList<>();
            for (Long serviceId : serviceIds) {
                Favorite favorite = new Favorite(null, Favorite.Category.Tenant, tenant.getId(), serviceId, 0);
                favorites.add(favorite);
            }
            if (!favorites.isEmpty()) {
                for (Favorite favorite : favorites) {
                    favoriteRepository.save(favorite);
                }
            }
            // 获取配额信息
            ArrayList<ResourceQuota> resourceQuotas = new ArrayList<>();
            if (tenant.getQuotaAble()) {
                List<MetaBean> metas = tenantModel.getMetas();
                for (MetaBean meta : metas) {
                    ResourceQuota metaQuota = new ResourceQuota(tenant.getId(), ResourceQuota.Catalog.TENANT,
                            meta.getQuota(), 0L, 0L, meta.getUnit(), meta.getCode(), ResourceQuota.Category.Meta);
                    resourceQuotas.add(metaQuota);
                }
                List<ServiceBean> services = tenantModel.getServices();
                for (ServiceBean service : services) {
                    if (StringUtils.isNotBlank(service.getUnit())) {
                        ResourceQuota metaQuota =
                                new ResourceQuota(tenant.getId(), ResourceQuota.Catalog.TENANT, service.getQuota(), 0L, 0L,
                                        service.getUnit(), service.getCode(), ResourceQuota.Category.Service);
                        resourceQuotas.add(metaQuota);
                    }
                }
            }
            tenantGrantMessage.setAddQuota(resourceQuotas);
            amqpTemplate.convertAndSend(TENANT_GRANT_QUEUE, tenantGrantMessage);
            log.info("send tenant grant message to cos....");
            // 资源池处理
            List<Long> poolIds = Optional.ofNullable(tenantModel.getGroupIds()).orElse(new ArrayList<>());
            if (!ListTool.isEmpty(poolIds)) {
                List<TenantPool> pools = Lists.newArrayList();
                for (Long poolId : poolIds) {
                    pools.add(new TenantPool(tenant.getId(), poolId));
                }
                if (!ListTool.isEmpty(pools)) {
                    boolean flag = tenantPoolRepository.batchSave(pools, pools.size());
                    if (!flag) {
                        return new GeneralResult<>(false, "资源池配置失败");
                    } else {
                        // 创建租户项目
                        taskExecutor.execute(() -> cmpInternalService.createOpProject(poolIds, tenant.getId(),
                                requestContext.getApiKey()));
                    }
                }
            }
            return new Result(true, "租户创建成功");
        } catch (Exception e) {
            log.error("Create Tenant fail:", e);
            throw new InternalServerException("租户创建失败", e, e.getMessage());
        }
    }

    /**
     * 回滚租户脏数据
     *
     * @param id
     * @param requestContext
     * @throws Exception
     */
    private void callback(Long id, RequestContext requestContext) {
        try {
            // 删除租户信息
            tenantRepository.delete(id, requestContext.getTarget());
            // 删除租户随机数
            securityRepository.delete(id, Catalog.Tenant);
        } catch (Exception e) {
            log.error("call back tenant data error,", e);
        }
    }

    /**
     * 修改租户
     *
     * @param tenantModel    租户信息json串
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional
    public Result modify(Long id, TenantModel tenantModel, RequestContext requestContext) {
        Assert.notNull(tenantModel, "TenantBean can't be null");
        tenantModel.setName(tenantModel.getName().trim());
        String path = Tenant.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户不存在");
            if (StringUtils.isNotBlank(tenantModel.getGroupCode())
                    && !tenantModel.getGroupCode().equals(tenant.getGroupCode())) {
                Tenant existName = tenantRepository.getByCode(tenantModel.getGroupCode());
                Assert.isNull(existName, "该租户编码已存在");
            }
            if (tenantModel.getEmail() != null && !tenantModel.getEmail().equals(tenant.getEmail())) {
                Tenant existName = tenantRepository.getByEmail(tenantModel.getEmail());
                Assert.isNull(existName, "该邮箱已被注册");
            }
            // 如果组织不为空，同时与库中存储不对应，同步更新租户下所有用户组织对应关系
            boolean isUpdateUser = false;
            Long departId = null;
            String departIds = null;
            if (tenant.getDepartId() == null
                    || (tenantModel.getDepartId() != null && tenant.getDepartId() != null
                    && !tenant.getDepartId().equals(tenantModel.getDepartId())) // 租户更新组织
                    || (tenant.getDepartId() != null && tenantModel.getDepartId() == null) // 更新为null，数据库中不为null
            ) {
                isUpdateUser = true;
                departId = tenantModel.getDepartId() == null ? null : tenantModel.getDepartId();
                departIds = tenantModel.getDepartIds() == null ? null : String.valueOf(tenantModel.getDepartIds());
            }

            BeanUtils.copyProperties(tenantModel, tenant, "password");
            tenant.setMenderId(requestContext.getTarget());
            if (tenant.getExpiredDate() != null && tenant.getExpiredDate().before(new Date())) {
                tenant.setStatus(BocloudStatus.Basic.EXPIRED.name());
            } else {
                if (tenant.getStatus().equals(BocloudStatus.Basic.EXPIRED.name())) {
                    tenant.setStatus(BocloudStatus.Basic.NORMAL.name());
                }
            }
            boolean result = tenantRepository.update(tenant);
            while (isUpdateUser) {
                boolean res = userRepository.updateDepartIdByTenantId(tenant.getId(), departId, departIds);
                isUpdateUser = false;
            }
            // 缓存更新租户姓名
            redisTemplate.opsForValue().set(Catalog.Tenant.name() + "_Name_" + tenant.getId(), tenant.getName());
            return result ? new Result(true, "修改租户成功") : new Result(false, "修改租     户失败");
        } catch (Exception e) {
            log.error("Modify Tenant fail:", e);
            throw new InternalServerException("修改租户失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 删除租户
     *
     * @param id             租户id
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id, RequestContext requestContext) throws InternalServerException {
        String path = Tenant.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户信息不存在");
            // 校验该租户下是否有正在使用的服务资源
            GeneralResult result = cosInternelService.serviceQuotaCheck(id, Catalog.Tenant.name());
            Assert.isTrue(result.isSuccess(), result.getMessage());
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap(Common.TENANTID, tenant.getId()), Sign.EQ));
            // 判断是否存在用户
            int count = this.userRepository.count(params, null);
            Assert.isTrue(0 == count, "该租户下存在用户");
            ArrayList<Param> tenantParam = new ArrayList<>();
            tenantParam.add(new Param(MapTools.simpleMap(Common.PARENTID, tenant.getId()), Sign.EQ));
            Assert.isTrue(tenantRepository.count(tenantParam) == 0, "该租户下存在子租户");
            Assert.isTrue("LOGOUT".equals(tenant.getStatus()), "只有注销租户可删除");
            // 删除配额
            cosQuotaService.delete(id, QuotaTargetCatalog.TENANT.name());
            // 删除租户已经授权的服务
            cosInternelService.deleteByTenant(tenant.getId());
            // 删除租户角色
            roleRepository.deleteByTenant(tenant.getId(), requestContext.getTarget());
            this.cosInternelService.deleteByTenant(id);
            // this.cosInternelService.deleteByTenantId(requestContext.getApiKey(), id);
            // 删除租户
            tenantRepository.delete(id, requestContext.getTarget());
            // 删除租户随机数
            this.securityRepository.delete(id, Catalog.Tenant);
            // 删除租户缓存数据
            redisTemplate.delete(Catalog.Tenant.name() + "_" + tenant.getId());
            // redisTemplate.delete(Catalog.Tenant.name() + "_Name_" + tenant.getId());
            // 删除授信账户信息
            cosInternelService.deleteTenantAccount(id, requestContext.getApiKey());
            // 删除租户token
            String tokenKey = Common.TOKEN + "_" + Catalog.Tenant.name() + "_" + id;
            redisTemplate.delete(tokenKey + "_n");
            redisTemplate.delete(tokenKey + "_o");
            return new Result(true, "删除租户成功");
        } catch (Exception e) {
            log.error("remove tenant fail:", e);
            throw new InternalServerException("删除租户失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 查询租户详情
     *
     * @param id 租户id
     * @return
     */
    @Override
    public GeneralResult<Tenant> detail(Long id) {
        try {
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "tenant data don't exist");
            Department department = departmentRepository.query(tenant.getDepartId());
            if (department != null) {
                tenant.setDepartmentName(department.getName());
            }
            List<PoolGroup> poolGroups = poolGroupRepository.listTenantPoolGroup(id);
            Set<String> vendors = poolGroups.stream().map(PoolGroup::getVendorType).collect(Collectors.toSet());
            tenant.setVendors(vendors);
            return new GeneralResult<>(true, tenant, "查询租户详情成功");
        } catch (Exception e) {
            log.error("query tenant detail fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 递归查询子节点ID
     *
     * @param childrenId  子节点id集合
     * @param permissions 目标权限集合
     * @return
     */
    @SuppressWarnings("unused")
    private List<Long> childrenIds(List<Long> childrenId, List<Permission> permissions) {
        try {
            // 目标权限为空 返回子权限id集合
            if (null == permissions || permissions.isEmpty()) {
                return childrenId;
            }
            List<Permission> children = new ArrayList<>();
            for (Permission permission : permissions) {
                childrenId.add(permission.getId());
                // 查询目标权限子权限集合
                List<Permission> child = permissionRepository.list(permission.getId(), permission.getTenant());
                children.addAll(child);
            }
            if (children.isEmpty()) {
                // 目标权限子权限为空则直接返回
                return childrenId;
            } else {
                // 目标权限子权限不为空，再次递归，获取子权限的子权限
                return this.childrenIds(childrenId, children);
            }
        } catch (Exception e) {
            log.error("get permission childrenIds fail:", e);
            return childrenId;
        }
    }

    /**
     * 分页查询租户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Override
    public GeneralResult<GridBean<Tenant>> list(Pager pager, RequestContext requestContext) {
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            List<Tenant> list;
            int total;
            GridBean<Tenant> gridBean;
            List<SimpleBean> beans;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtCreate", Common.ONE);
            sorter.put("id", Common.ONE);
            total = tenantRepository.count(params);
            Map<Long, Department> departmentMap = departmentRepository.listAll().stream()
                    .collect(Collectors.toMap(Department::getId, Function.identity()));
            if (pager.getSimple()) {
                beans = tenantRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = tenantRepository.list(pager.getPage(), pager.getRows(), params, sorter);
                for (Tenant tenant : list) {
                    Department department = departmentMap.get(tenant.getDepartId());
                    if (department != null) {
                        tenant.setDepartmentName(department.getName());
                        List<Long> deptIdTree = getDeptIdTree(departmentMap, department);
                        Collections.reverse(deptIdTree);
                        String string = JSONObject.toJSONString(deptIdTree);
                        tenant.setDepartIds(string);
                    } else {
                        tenant.setDepartIds("[]");
                    }
                }
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询租户成功");
        } catch (Exception e) {
            log.error("list tenant fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private List<Long> getDeptIdTree(Map<Long, Department> departmentMap, Department department) {
        List<Long> result = new ArrayList<>();
        result.add(department.getId());
        Department parent = departmentMap.get(department.getParentId());
        if (parent != null) {
            List<Long> deptIdTree = getDeptIdTree(departmentMap, parent);
            result.addAll(deptIdTree);
        }
        return result;
    }

    @Override
    public Result exist(String account) {
        try {
            Tenant existAccount = tenantRepository.getByAccount(account);
            return existAccount == null ? new Result(true, "账号可以使用") : new Result(false, "账号已经存在");
        } catch (Exception e) {
            log.error("get tenant info error,", e);
            return new Result(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Tenant>> listTenantByCity(String city) {
        try {
            List<Tenant> list = tenantRepository.listByCity(city);
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 根据资源池id删除租户资源池关系
     *
     * @param poolId
     * @return
     */
    @Override
    public Result deleteByPoolId(Long poolId) {
        String path = TenantPool.class.getSimpleName() + "_pool_" + poolId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            boolean result = tenantPoolRepository.deletePoolByGroupId(poolId);
            if (result) {
                return new Result(true, "租户资源池关系删除成功");
            } else {
                return new Result(false, "租户资源池关系删除失败");
            }
        } catch (Exception e) {
            log.error("remove Tenant pool fail:", e);
            return new Result(false, "租户资源池关系删除失败");
        }
    }

    /**
     * 租户配置资源池
     *
     * @param id
     * @param category
     * @param poolIds
     * @param requestContext
     * @return
     */
    @Override
    public Result configPool(Long id, String category, List<Long> poolIds, RequestContext requestContext) {
        try {
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户不存在");
            List<Long> removePoolIds = new ArrayList<>();
            List<TenantPool> tenantPools = tenantPoolRepository.listTenantPool(id);
            if (ListTool.isEmpty(poolIds)) {
                for (TenantPool tenantPool : tenantPools) {
                    removePoolIds.add(tenantPool.getPoolGroupId());
                }
            } else {
                for (TenantPool pool : tenantPools) {
                    if (!poolIds.contains(pool.getPoolGroupId())) {
                        removePoolIds.add(pool.getPoolGroupId());
                    }
                }
            }
            if (!ListTool.isEmpty(removePoolIds)) {
                // 校验资源池下是否有资源正在使用
                GeneralResult result = cmpInternalService.tenantCheck(removePoolIds, id);
                if (result.isFailed()) {
                    result.setMessage("该租户" + result.getMessage());
                    return result;
                }
            }
            List<Tenant> tenants = tenantRepository.listChildren(id);
            for (Tenant t : tenants) {
                for (Long poolId : removePoolIds) {
                    List<TenantPool> pools = tenantPoolRepository.listGroupPool(t.getId(), poolId);
                    if (!ListTool.isEmpty(pools)) {
                        String poolName = "";
                        GeneralResult GeneralResult = cmpInternalService.detailPool(poolId);
                        if (GeneralResult.isSuccess()) {
                            JSONObject object =
                                    JSONObject.parseObject(JSONObject.toJSONString(GeneralResult.getData()));
                            poolName = object.getString("name");
                        }
                        return new Result(false, "资源池【" + poolName + "】正在被子租户使用，不能移除");
                    }
                }
            }
            // 删除旧数据
            tenantPoolRepository.deleteTenantPool(id);
            if (!ListTool.isEmpty(poolIds)) {
                List<TenantPool> pools = Lists.newArrayList();
                for (Long poolId : poolIds) {
                    pools.add(new TenantPool(id, poolId));
                }
                if (!ListTool.isEmpty(pools)) {
                    boolean flag = tenantPoolRepository.batchSave(pools, pools.size());
                    if (!flag) {
                        return new GeneralResult<>(false, "资源池配置失败");
                    } else {
                        // 创建租户项目
                        taskExecutor.execute(() -> cmpInternalService.createOpProject(poolIds, tenant.getId(),
                                requestContext.getApiKey()));
                    }
                }
            }
            return new Result(true, "资源池配置成功");
        } catch (Exception e) {
            log.error("Tenant config fail:", e);
            return new Result(false, "资源池配置失败");
        }
    }

    /**
     * 根据租户查询资源池关联信息
     *
     * @param id
     * @return
     */
    @Override
    public GeneralResult<List<TenantPool>> listTenantPool(Long id) {
        try {
            List<TenantPool> pools = tenantPoolRepository.listTenantPool(id);
            return new GeneralResult<>(true, pools, "查询成功");
        } catch (Exception e) {
            log.error("query Tenant pool fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 租户配置业务
     *
     * @param id
     * @param businessIds
     * @return
     */
    @Override
    public Result configBusiness(Long id, List<Long> businessIds) {
        try {
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户不存在");
            List<Long> removeBusinessIds = new ArrayList<>();
            List<Business> tenantBusinesses = businessRepository.listBusiness(id);
            if (ListTool.isEmpty(businessIds)) {
                for (Business tenantBusiness : tenantBusinesses) {
                    removeBusinessIds.add(tenantBusiness.getId());
                }
            } else {
                for (Business tenantBusiness : tenantBusinesses) {
                    if (!businessIds.contains(tenantBusiness.getId())) {
                        removeBusinessIds.add(tenantBusiness.getId());
                    }
                }
            }
            // 验证本租户有没有在使用要移除的业务
            for (Long businessId : removeBusinessIds) {
                // 判断业务有没有被项目使用
                List<Project> projects = projectRepository.listByBusiness(businessId, id);
                if (!ListTool.isEmpty(projects)) {
                    String businessName = "";
                    Business business = businessRepository.query(businessId);
                    if (null != business) {
                        businessName = business.getName();
                    }
                    return new Result(false, "业务【" + businessName + "】正在被该租户下项目使用，不能移除");
                }
            }
            // 验证租户有没有在使用要移除的业务
            List<Tenant> tenants = tenantRepository.listChildren(id);
            for (Tenant t : tenants) {
                for (Long businessId : removeBusinessIds) {
                    List<Business> businesses = businessRepository.listTenantBusiness(t.getId(), businessId);
                    if (!ListTool.isEmpty(businesses)) {
                        String businessName = "";
                        Business business = businessRepository.query(businessId);
                        if (null != business) {
                            businessName = business.getName();
                        }
                        return new Result(false, "业务【" + businessName + "】正在被租户【" + t.getName() + "】使用，不能移除");
                    }
                }
            }
            // 删除旧数据
            tenantPoolRepository.deleteTenantPool(id);
            if (!ListTool.isEmpty(businessIds)) {
                List<Business> businesses = Lists.newArrayList();
                for (Long businessId : businessIds) {
                    businesses.add(new Business(businessId, "business" + tenant.getAccount(), 0, id));
                }
                if (!ListTool.isEmpty(businesses)) {
                    boolean flag = businessRepository.batchSave(businesses, businesses.size());
                    if (!flag) {
                        return new Result(false, "业务配置失败");
                    }
                }
            }
            return new Result(true, "业务配置成功");
        } catch (Exception e) {
            log.error("Business config fail:", e);
            return new Result(false, "业务配置失败");
        }
    }

    /**
     * 租户登录
     *
     * @param account
     * @param password
     * @param sessionId
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public GeneralResult<Map<String, Object>> login(String account, String password, String sessionId, String requestIp)
            throws InternalServerException {
        LoginRecord loginRecord = new LoginRecord();
        loginRecord.setRequestIp(requestIp);
        loginRecord.setResponseIp(currentService.getService().getHost());
        loginRecord.setAccount(account);
        loginRecord.setCategory(LoginRecord.Category.Tenant);
        loginRecord.setGmtCreate(new Date());
        // 判断租户是否存在
        Tenant tenant;
        AccountSecurity security;
        try {
            tenant = tenantRepository.getByAccount(account);
            if (null == tenant) {
                log.warn("Tenant is null");
                handleLoginLog(false, "账号或者密码错误", loginRecord);
                return new GeneralResult<>(false, "账号或密码错误");
            }
            security = securityRepository.getByTarget(tenant.getId(), Catalog.Tenant);
            if (null == security) {
                handleLoginLog(false, "账号或者密码错误", loginRecord);
                log.warn("Security is null");
                return new GeneralResult<>(false, "账号或密码错误");
            }

            Encryptor encryptor = new AESEncryptor();
            // 对前端传过来的密码进行解密
            password = encryptor.decrypt(password.trim(), null);
            if (!StringUtils.isNotBlank(password)) {
                handleLoginLog(false, "账号或密码错误", loginRecord);
                return new GeneralResult<>(false, "账号或密码错误");
            }
            // 密码加密
            encryptor = new SHAEncryptor();
            String encrypt = encryptor.encrypt(password, security.getSalt());
            // 判断是否密码正确
            if (null == encrypt || !encrypt.equals(tenant.getPassword())) {
                handleLoginLog(false, "账号或者密码错误", loginRecord);
                log.warn("Wrong account or password");
                return new GeneralResult<>(false, "账号或密码错误");
            }
        } catch (Exception e) {
            handleLoginLog(false, "账号错误", loginRecord);
            log.error("Get tenant fail by account:", e);
            return new GeneralResult<>(false, "账号或密码错误");
        }
        // 更新租户登录状态
        try {
            GeneralResult<Map<String, Object>> result = statusCheck(loginRecord, tenant);
            if (result.isFailed()) {
                return result;
            }
            tenant.setLoginStatus(true);
            tenant.setSessionId(sessionId);
            // 获取上一次登录时间 并重置上一次登录时间为现在
            Date lastLoginDate = tenant.getLastLoginDate();
            tenant.setLastLoginDate(new Date());
            tenantRepository.update(tenant);
            tenant.setLastLoginDate(lastLoginDate);
        } catch (Exception e) {
            handleLoginLog(false, "更新租户登录状态失败", loginRecord);
            log.error("Update tenant login status fail：", e);
            throw new InternalServerException("更新租户登录状态失败", e, e.getMessage());
        }
        // 刷新权限缓存
        flushRoleApiPermission();
        // 构建返回数据
        Map<String, Object> map = getLoginResult(loginRecord, tenant, security);
        if (map != null) {
            handleLoginLog(true, "租户登录成功", loginRecord);
            return new GeneralResult<>(true, map, "租户登录成功");
        } else {
            handleLoginLog(false, "租户登录失败", loginRecord);
            return new GeneralResult<>(false, "登录失败");
        }
    }

    @Override
    public GeneralResult checkPassword(RequestContext requestContext, String password) {
        AccountSecurity security;
        Tenant tenant;
        try {
            tenant = tenantRepository.query(requestContext.getTenant());
            security = securityRepository.getByTarget(requestContext.getTenant(), Catalog.Tenant);
            Encryptor encryptor = new AESEncryptor();
            // 对前端传过来的密码进行解密
            password = encryptor.decrypt(password.trim(), null);
            if (!StringUtils.isNotBlank(password)) {
                return new GeneralResult<>(false, "账号或密码错误");
            }
            // 密码加密
            encryptor = new SHAEncryptor();
            String encrypt = encryptor.encrypt(password, security.getSalt());
            // 判断是否密码正确
            if (null == encrypt || !encrypt.equals(tenant.getPassword())) {
                log.warn("Wrong account or password");
                return new GeneralResult<>(false, "账号或密码错误");
            }
        } catch (Exception e) {
            log.error("密码解密异常{}", e.getMessage());
            return new GeneralResult<>(false, "账号或密码错误");
        }
        return new GeneralResult(true, "密码正确");
    }

    /**
     * 获取登录结果及token处理
     *
     * @param loginRecord
     * @param tenant
     * @param security
     * @return
     */
    private Map<String, Object> getLoginResult(LoginRecord loginRecord, Tenant tenant, AccountSecurity security) {
        try {
            Map<String, Object> map = MapTools.simpleMap(Common.TENANT, tenant);
            map.put(Common.API_KEY, security.getApiKey());
            map.put("accountCategory", Catalog.Tenant.name());
            // 构建token
            Map<String, Object> header = MapTools.simpleMap("alg", SignatureAlgorithm.HS256.getValue());
            header.put("typ", "JWT");
            Map<String, Object> payload = MapTools.simpleMap(Common.ACCOUNT, tenant.getAccount());
            payload.put("catalog", Catalog.Tenant.name());
            payload.put(Common.API_KEY, security.getApiKey());
            payload.put(Common.UUID, tenant.getId());
            payload.put(Common.NAME, tenant.getName());
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH, 3);
            // 加密jwt生成token并存入session和cookie
            String token = Tokens.generate(header, payload, calendar.getTime());
            map.put(Common.TOKEN, token);
            String tokenKey = "token_" + Catalog.Tenant.name() + "_" + tenant.getId();
            this.redisTemplate.delete(tokenKey + "_o");
            HashMap<String, Object> tokenJson = new HashMap<>();
            tokenJson.put("token", token);
            tokenJson.put("refreshTime", LocalDateTime.now().plusMinutes(10).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            redisTemplate.opsForHash().putAll(tokenKey + "_n", tokenJson);
            this.redisTemplate.expire(tokenKey + "_n", 30L, TimeUnit.MINUTES);
            return map;
        } catch (Exception e) {
            log.error("get login result error,", e);
            return null;
        }
    }

    private GeneralResult<Map<String, Object>> statusCheck(LoginRecord log, Tenant tenant) {
        if (tenant.getStatus().equals(BaseStatus.ABNORMAL.name())) {
            handleLoginLog(false, "账号已冻结", log);
            return new GeneralResult<>(false, "冻结账号无法登录");
        }
        if (tenant.getStatus().equals(BocloudStatus.Basic.LOGOUT.name())) {
            handleLoginLog(false, "账号已注销", log);
            return new GeneralResult<>(false, "注销账号无法登录");
        }
        if (tenant.getStatus().equals(BocloudStatus.Basic.EXPIRED.name())) {
            handleLoginLog(false, "账号已过期", log);
            return new GeneralResult<>(false, "账号已过期，请联系管理员");
        }
        return new GeneralResult<>(true, "校验通过");
    }

    private void flushRoleApiPermission() {
        try {
            List<Long> roleIds = roleRepository.getTenantRole().stream().map(Role::getId).collect(Collectors.toList());
            List<ApiPermission> apiPermissions = apiPermissionRepository.listRoleApiPermission(roleIds);
            Map<Long, List<ApiPermission>> rolePermissionMap =
                    apiPermissions.stream().collect(Collectors.groupingBy(ApiPermission::getRoleId));
            for (Map.Entry<Long, List<ApiPermission>> rolePermissionEntry : rolePermissionMap.entrySet()) {
                Long roleId = rolePermissionEntry.getKey();
                Map<String, List<ApiPermission>> permissionMap =
                        rolePermissionEntry.getValue().stream().collect(Collectors.groupingBy(ApiPermission::getModule));
                for (Map.Entry<String, List<ApiPermission>> permission : permissionMap.entrySet()) {
                    String module = permission.getKey();
                    List<String> patterns = permission.getValue().stream()
                            .map(s -> new String(Base64.getDecoder().decode(s.getPattern()))).collect(Collectors.toList());
                    if (!patterns.isEmpty()) {
                        redisTemplate.opsForHash().put("api_permission_" + roleId, module,
                                JSONObject.toJSONString(patterns));
                    }
                }
            }
        } catch (Exception e) {
            log.error("flush api permission error,", e);
        }
    }

    /**
     * 保存租户登录日志
     *
     * @param status 登录状态
     * @param detail 登录详情
     * @param record 日志对象
     */
    private void handleLoginLog(boolean status, String detail, LoginRecord record) {
        record.setStatus(status);
        record.setDetail(detail);
        try {
            this.save(record);
        } catch (Exception e) {
            log.error("save login record error");
        }
    }

    /**
     * 租户退出
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result logout(Long id) throws InternalServerException {
        try {
            Tenant tenant = tenantRepository.query(id);
            if (tenant != null) {
                tenant.setLoginStatus(false);
                tenant.setSessionId("");
                tenantRepository.update(tenant);
                String tokenKey = Common.TOKEN + "_" + Catalog.Tenant.name() + "_" + id;
                redisTemplate.delete(tokenKey + "_n");
                redisTemplate.delete(tokenKey + "_o");
            }
            return new Result(true, "租户退出成功!");
        } catch (Exception e) {
            log.error("get user fail:", e);
            throw new InternalServerException("获取租户信息失败!", e, e.getMessage());
        }
    }

    @Override
    @Transactional
    public Result updateStatus(Long id, ActionModel.Action action, RequestContext requestContext) {
        String path = Tenant.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            switch (action) {
                case lock:
                    boolean result = tenantRepository.lock(id, requestContext.getTarget());
                    this.logout(id);
                    // 查找租户下的用户
                    List<User> lockUser = userRepository.listByTenant(id);
                    if (!lockUser.isEmpty()) {
                        userRepository.batchLock(lockUser, requestContext.getTarget());
                    }
                    String userToken = Common.TOKEN + "_" + Catalog.Tenant.name() + "_" + id;
                    redisTemplate.delete(userToken + "_n");
                    redisTemplate.delete(userToken + "_o");
                    return result ? new Result(true, "冻结租户成功") : new Result(false, "冻结租户失败");
                case active:
                    result = tenantRepository.active(id, requestContext.getTarget());
                    this.logout(id);
                    // 查找租户下的用户
                    List<User> activeUser = userRepository.listByTenant(id);
                    if (!activeUser.isEmpty()) {
                        userRepository.batchActive(activeUser, requestContext.getTarget());
                    }
                    return result ? new Result(true, "解冻租户成功") : new Result(false, "解冻租户失败");
                case logout:
                    GeneralResult serviceQuotaCheck = cosInternelService.serviceQuotaCheck(id, Catalog.Tenant.name());
                    Assert.isTrue(serviceQuotaCheck.isSuccess(), serviceQuotaCheck.getMessage() + ",无法注销。");
                    Integer tenantCoupons = cosInternelService.tenantCoupons(id, null);
                    Assert.isTrue(0 == tenantCoupons, "租户下存在优惠券,无法注销");
                    List<User> tenantUsers = userRepository.listByTenant(id);
                    Assert.isTrue(tenantUsers.isEmpty(), "租户下存在用户,无法注销。");
                    List<Tenant> childTenant = tenantRepository.listChildren(id);
                    Assert.isTrue(childTenant.isEmpty(), "租户下存在子租户,无法注销。");
                    // 资源判断
                    result = tenantRepository.logout(id, requestContext.getTarget());
                    String tokenKey = Common.TOKEN + "_" + Catalog.Tenant.name() + "_" + id;
                    redisTemplate.delete(tokenKey + "_n");
                    redisTemplate.delete(tokenKey + "_o");
                    return result ? new Result(true, "注销租户成功") : new Result(false, "注销租户失败");

                default:
                    break;
            }
            return new Result(true, "修改用户状态成功");
        } catch (Exception e) {
            log.error("modify tenant status error,", e);
            return GeneralResult.FAILED("修改用户状态失败:" + e.getMessage());
        }
    }

    @Override
    public Result changePwd(Long id, String oldPassword, String password, RequestContext requestContext) {
        String path = Tenant.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String salt = UUID.randomUUID().toString();
            SHAEncryptor sha = new SHAEncryptor();
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户不存在");
            Encryptor encryptor = new AESEncryptor();
            oldPassword = encryptor.decrypt(oldPassword.trim(), null);
            Assert.isTrue(StringUtils.isNotBlank(oldPassword), "旧密码不存在");
            // 校验旧密码
            Result checkResult = this.check(id, oldPassword);
            if (checkResult.isFailed()) {
                return checkResult;
            }
            // 对前端传过来的密码进行解密
            password = encryptor.decrypt(password.trim(), null);
            Assert.isTrue(StringUtils.isNotBlank(password), "新密码不存在");
            String encrypt = sha.encrypt(password, salt);
            tenant.setPassword(encrypt);
            tenant.setLastPwdModifyDate(new Date());
            tenantRepository.update(tenant);
            // 准备随机数数据
            AccountSecurity security = securityRepository.getByTarget(tenant.getId(), Catalog.Tenant);
            security.setSalt(salt);
            // 更新随机数
            securityRepository.update(security);
            return new Result(true, "修改密码成功");
        } catch (Exception e) {
            log.error("modify tenant password error,", e);
            return GeneralResult.FAILED("修改密码失败:" + e.getMessage());
        }
    }

    @Override
    public Result resetPwd(Long id, String password, RequestContext requestContext) {
        String path = Tenant.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String salt = UUID.randomUUID().toString();
            SHAEncryptor sha = new SHAEncryptor();
            Tenant tenant = tenantRepository.query(id);
            Assert.notNull(tenant, "租户不存在");
            tenant.setPassword(sha.encrypt(new AESEncryptor().decrypt(password.trim(), null), salt));
            tenant.setMenderId(requestContext.getTarget());
            tenant.setLastPwdModifyDate(new Date());
            tenantRepository.update(tenant);
            // 准备随机数数据
            AccountSecurity tenantSecurity = securityRepository.getByTarget(tenant.getId(), Catalog.Tenant);
            tenantSecurity.setSalt(salt);
            // 更新随机数
            securityRepository.update(tenantSecurity);
            return new Result(true, "重置密码成功");
        } catch (Exception e) {
            log.error("modify tenant password error,", e);
            return GeneralResult.FAILED("重置密码失败:" + e.getMessage());
        }
    }

    /**
     * 校验密码
     *
     * @param id
     * @param password
     * @return
     */
    @Override
    public Result check(Long id, String password) {
        try {
            Tenant tenant = tenantRepository.query(id);
            AccountSecurity security = securityRepository.getByTarget(id, Catalog.Tenant);
            Assert.isTrue(null != tenant && null != security, "原密码验证失败");
            SHAEncryptor sha = new SHAEncryptor();
            String encrypt = sha.encrypt(password, security.getSalt());
            Assert.isTrue((null != encrypt && encrypt.equals(tenant.getPassword())), "原密码验证失败");
            return new Result(true, "原密码正确");
        } catch (Exception e) {
            log.error("old password not correct:", e);
            return new Result(false, e.getMessage());
        }
    }

    /**
     * 检查租户状态
     *
     * @param id
     * @param token
     * @return
     */
    @Override
    public Result checkTenant(Long id, String token) {
        try {
            Tenant tenant = tenantRepository.query(id);
            if (null != tenant) {
                if (BaseStatus.ABNORMAL.name().equals(tenant.getStatus())) {
                    return new Result(false, "此租户已被冻结!");
                }
                if ("LOGOUT".equals(tenant.getStatus())) {
                    return new Result(false, "此租户已被注销!");
                }
                String tokenKey = Common.TOKEN + "_" + Catalog.Tenant + "_" + id;
                String newTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_n");
                if (!token.equals(newTokenCache)) {
                    return GeneralResult.FAILED("此租户已在别处登录!");
                }
                return new Result(true, "");
            } else {
                return new Result(false, "该租户信息不存在!");
            }
        } catch (Exception e) {
            log.error("check tenant login fail:", e);
            return new Result(false, "检查租户登录失败!");
        }
    }

    /**
     * 查询租户的角色
     *
     * @param tenantId
     * @return
     */
    @Override
    public GeneralResult<List<Role>> listRoles(Long tenantId) {
        try {
            List<Role> roles = this.roleRepository.listByTenant(tenantId);
            Assert.notEmpty(roles, "查询所有角色失败!");
            return new GeneralResult<>(true, roles, "查询成功");
        } catch (Exception e) {
            log.error("list permission by tenant id:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取apiKey
     *
     * @param apiKey
     * @return
     */
    @Override
    public GeneralResult<AccountSecurity> getByApiKey(String apiKey) {
        try {
            AccountSecurity security = securityRepository.getByApiKey(apiKey, Catalog.Tenant);
            return new GeneralResult<>(true, security, "获取安全密钥成功");
        } catch (Exception e) {
            log.error("get secKey fail:", e);
            return new GeneralResult<>(false, "获取安全密钥失败");
        }
    }

    /**
     * 根据租户账号冻结租户基本信息
     *
     * @param account
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result lockByAccount(String account) throws InternalServerException {
        String path;
        boolean result;
        Tenant tenant;
        try {
            tenant = tenantRepository.getByAccount(account);
            Assert.notNull(tenant, "租户不存在");
        } catch (Exception e) {
            log.error("get tenant data fail:", e);
            return new Result(false, "获取租户信息失败:" + e.getMessage());
        }
        path = Tenant.class.getSimpleName() + "_" + tenant.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            result = tenantRepository.lock(tenant.getId(), 1L);
            return result ? new Result(true, "冻结租户成功") : new Result(false, "冻结租户失败");
        } catch (Exception e) {
            log.error("freeze tenant fail:", e);
            throw new InternalServerException("冻结租户失败," + e.getMessage(), e, e.getMessage());
        }
    }

    @Override
    public Result batchLock(List<Long> ids, RequestContext context) {
        try {
            boolean result = tenantRepository.batchLock(ids, context.getTarget());
            for (Long id : ids) {
                String tokenKey = Common.TOKEN + "_" + Catalog.Tenant.name() + "_" + id;
                redisTemplate.delete(tokenKey + "_n");
                redisTemplate.delete(tokenKey + "_o");
            }
            return result ? new Result(true, "批量冻结租户成功") : new Result(false, "批量冻结租户失败");
        } catch (Exception e) {
            log.error("batch locked tenants fail:", e);
            return new Result(false, "批量冻结租户失败," + e.getMessage());
        }
    }

    @Override
    public Result batchActive(List<Long> ids, RequestContext context) {
        try {
            boolean result = tenantRepository.batchActive(ids, context.getTarget());
            return result ? new Result(true, "批量解冻租户成功") : new Result(false, "批量解冻租户失败");
        } catch (Exception e) {
            log.error("batch active tenants fail:", e);
            return new Result(false, "批量解冻租户失败," + e.getMessage());
        }
    }

    @Override
    public Result batchRemove(List<Long> ids, RequestContext context) {
        for (Long id : ids) {
            this.remove(id, context);
        }
        return new Result(true, "批量刪除租戶成功");
    }

    /**
     * 保存租户登录数据
     *
     * @param loginRecord
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result save(LoginRecord loginRecord) throws InternalServerException {
        String path = LoginRecord.class.getSimpleName() + "_" + loginRecord.getAccount();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            loginRecordRepository.save(loginRecord);
            return new Result(true, "租户登录日志记录成功！");
        } catch (Exception e) {
            log.error("save tenant login log fail:", e);
            throw new InternalServerException("租户登录日志记录失败", e, e.getMessage());
        }
    }

    /**
     * 租户登录记录列表展示
     *
     * @param pager
     * @return
     */
    @Override
    public GeneralResult<GridBean<LoginRecord>> list(Pager pager) {
        int total;
        GridBean<LoginRecord> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmt_create", Common.ONE);
            total = tenantRepository.loginTimes(params);
            List<LoginRecord> list =
                    loginRecordRepository.listLoginRecord(pager.getPage(), pager.getRows(), params, sorter, Catalog.Tenant);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query user login record list fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取租户权限
     *
     * @param context
     * @return
     */
    @Override
    public GeneralResult<Map<String, Object>> listPermissions(RequestContext context) {
        try {
            // 获取租户服务权限
            GeneralResult<List<PermissionBean>> servicePermission = getServicePermissionMap(context.getTarget());
            Assert.isTrue(servicePermission.isSuccess(), "获取租户菜单权限失败");
            Map<String, Object> resultMap = MapTools.simpleMap("permission", servicePermission.getData());
            // 返回对象中添加账号类型
            resultMap.put("accountCategory", context.getCatalog());
            return new GeneralResult<>(true, resultMap, "获取租户菜单权限成功");
        } catch (Exception e) {
            log.error("list by user id failed", e);
            return new GeneralResult<>(false, "查询失败!");
        }
    }

    /**
     * 获取租户服务菜单权限
     *
     * @param tenantId
     * @return
     */
    private GeneralResult<List<PermissionBean>> getServicePermissionMap(Long tenantId) {
        // 获取租户收藏（授权）的服务
        List<CloudServiceBean> favoriteServices =
                cloudServiceBeanRepository.getFavoriteServices(tenantId, RequestContext.Catalog.Tenant.name());
        // 获取租户收藏的服务编码集合
        List<String> permissionCodes =
                cloudServiceBeanRepository.getFavoriteServices(tenantId, RequestContext.Catalog.Tenant.name()).stream()
                        .map(CloudServiceBean::getCode).collect(Collectors.toList());
        // 获取所有自服务端权限
        List<Permission> allPermissions = permissionRepository.list(true);
        Map<Long, Permission> allPermissionMap =
                allPermissions.stream().collect(Collectors.toMap(Permission::getId, Function.identity()));
        List<Long> serviceIds = favoriteServices.stream().map(CloudServiceBean::getId).collect(Collectors.toList());
        // 获取授权的服务对应的菜单权限
        List<Permission> servicePermissions = permissionRepository.getServicesPermissions(serviceIds);
        HashMap<String, List<Permission>> servicePermissionMap = new HashMap<>(permissionCodes.size(), 0.75F);
        for (String code : permissionCodes) {
            ArrayList<Permission> permissions = new ArrayList<>();
            for (Permission servicePermission : servicePermissions) {
                if (servicePermission.getServiceCodes().contains(code)) {
                    permissions.add(servicePermission);
                }
            }
            servicePermissionMap.put(code, permissions);
        }
        HashMap<Long, Permission> resultMap = new HashMap<>(64, 0.75F);
        for (CloudServiceBean service : favoriteServices) {
            // 获取收藏的服务对应的菜单权限
            List<Permission> permissions = servicePermissionMap.get(service.getCode());
            for (Permission permission : permissions) {
                // 查询父菜单权限 (放入resultMap)
                getParents(allPermissionMap, resultMap, permission);
                // 获取当前服务权限相关的所有菜单权限（默认显示的和服务相关子权限）
                getRelatedPermission(resultMap, permission, permissionCodes, allPermissions);
                // 获取菜单权限父权限下默认显示权限(兄弟权限节点)
                List<Permission> brotherPermission = new ArrayList<>();
                Map<Long, List<Permission>> parentMap =
                        allPermissions.stream().collect(Collectors.groupingBy(Permission::getParentId));
                getBrotherPermission(permission.getParentId(), brotherPermission, parentMap);
                // 获取同级权限下默认显示权限
                List<Permission> alwaysPermissions = brotherPermission.stream()
                        .filter(permit -> "isAlwaysShow".equals(permit.getProps())).collect(Collectors.toList());
                // 使用map去重
                for (Permission alwaysPermission : alwaysPermissions) {
                    resultMap.put(alwaysPermission.getId(), alwaysPermission);
                }
            }
        }
        if (resultMap.isEmpty()) {
            return new GeneralResult<>(true, new ArrayList<>(), "获取服务菜单权限成功");
        }
        List<Permission> resultPermissions =
                resultMap.keySet().stream().map(resultMap::get).collect(Collectors.toList());
        // 重新装配权限集合
        List<PermissionBean> permissionBeans = resultPermissions.stream()
                .map(permit -> new PermissionBean(permit.getId(), permit.getName(), permit.getPath(), permit.getIcon(),
                        permit.getParentId(), permit.getPriority(), permit.getRouter(), permit.getComponent(),
                        permit.getRedirect(), permit.getMeta(), permit.getParams(), permit.getCategory(),
                        permit.getPluginsCode(), permit.getPluginsStatus(), permit.getButtonCode()))
                .collect(Collectors.toList());

        // license过滤列表
        Set<PermissionBean> resultPermission = new HashSet<>();
        List<Plugin> plugins = this.pluginService.runningPlugins(BoCloudService.CMP.getName(),
                BoCloudService.COS.getName(), BoCloudService.COP.getName());
        if (plugins == null || !validatePlugin) {
            // 对权限菜单进行递归排序
            List<PermissionBean> menus = sortPermission(0L, permissionBeans);
            return new GeneralResult<>(true, menus, "获取菜单权限信息成功");
        }
        permissionBeans.forEach(permissionBean -> {
            if (StringUtils.isBlank(permissionBean.getPluginsCode())) {
                resultPermission.add(permissionBean);
                return;
            }
            plugins.forEach(plugin -> {
                // 如果菜单绑定插件为空或者为开启状态，则进行添加操作
                boolean pluginsStatus = permissionBean.getPluginsStatus() != null && permissionBean.getPluginsStatus();
                if ((pluginsStatus && permissionBean.getPluginsCode().equals(plugin.getCode()))) {
                    resultPermission.add(permissionBean);
                }
            });
        });
        // 对权限菜单进行递归排序
        List<PermissionBean> menus = sortPermission(0L, new ArrayList<>(resultPermission));
        return new GeneralResult<>(true, menus, "获取菜单权限信息成功");
    }

    /**
     * 获取授权服务相关权限集合
     *
     * @param resultMap     返回集合
     * @param permission    目标权限
     * @param serviceCodes  服务编码
     * @param allPermission 自服务端权限集合
     */
    private void getRelatedPermission(Map<Long, Permission> resultMap, Permission permission, List<String> serviceCodes,
                                      List<Permission> allPermission) {
        // 递归获取目标权限的子权限及下属子权限
        List<Permission> childrenPermission = getChildrenPermissions(permission.getId(), allPermission);
        for (Permission childPermission : childrenPermission) {
            // 获取目标权限子权限中默认显示的权限及授权服务编码相关权限
            Map<Long,
                    Permission> permissionMap = childrenPermission.stream()
                    .filter(permit -> "isAlwaysShow".equals(permit.getProps())
                            || isContainsCode(serviceCodes, permit.getServiceCodes()))
                    .collect(Collectors.toMap(Permission::getId, Function.identity()));
            // 将符合的权限添加到返回集合中
            resultMap.putAll(permissionMap);
            // 对目标权限子权限进行递归，再次查询相关权限
            getRelatedPermission(resultMap, childPermission, serviceCodes, allPermission);
        }
        // 判断目标权限是否是授权服务相关权限或者默认显示权限
        if ("isAlwaysShow".equals(permission.getProps())
                || isContainsCode(serviceCodes, permission.getServiceCodes())) {
            resultMap.put(permission.getId(), permission);
        }
    }

    /**
     * 判断权限服务code字符串是否存在集合中
     *
     * @param serviceCodes
     * @param codesString
     * @return
     */
    private boolean isContainsCode(List<String> serviceCodes, String codesString) {
        return Optional.ofNullable(JSON.parseArray(codesString, String.class)).orElse(new ArrayList<>()).stream()
                .anyMatch(serviceCodes::contains);
    }

    /**
     * 对租户权限进行递归排序
     *
     * @param pid             父权限id
     * @param permissionBeans 需要排序的权限集合
     * @return
     */
    private List<PermissionBean> sortPermission(Long pid, List<PermissionBean> permissionBeans) {
        // 创建集合用于接收和返回递归后数据
        List<PermissionBean> newPermissions = Lists.newArrayList();
        for (PermissionBean permissionBean : permissionBeans) {
            if (null != permissionBean.getParentId() && permissionBean.getParentId().equals(pid)) {
                // 递归获取，获取子权限
                List<PermissionBean> children = sortPermission(permissionBean.getId(), permissionBeans);
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermitComparator());
                    permissionBean.setChildren(children);
                } else {
                    permissionBean.setChildren(null);
                }
                // 添加到返回集合
                newPermissions.add(permissionBean);
            }
        }
        newPermissions.sort(new PermitComparator());
        return newPermissions;
    }

    /**
     * 获取父权限
     *
     * @param parents    父权限集合
     * @param permission 目标权限
     * @return
     */
    private Map<Long, Permission> getParents(Map<Long, Permission> allPermissions, Map<Long, Permission> parents,
                                             Permission permission) {
        Long parentId = permission.getParentId();
        // 父权限id不存在时，递归出口，添加到父权限集合中，并返回
        if (null == parentId || 0L == parentId) {
            parents.put(permission.getId(), permission);
            return parents;
        } else {
            try {
                // 父权限id不为空，获取父权限，再次递归，并把当前权限添加到返回集合
                Permission parent = allPermissions.get(parentId);
                parents.put(permission.getId(), permission);
                return this.getParents(allPermissions, parents, parent);
            } catch (Exception e) {
                log.error("query permission parent list failure:", e);
                return null;
            }
        }
    }

    private void getBrotherPermission(Long pid, List<Permission> result, Map<Long, List<Permission>> permissions) {
        List<Permission> children = Optional.ofNullable(permissions.get(pid)).orElse(new ArrayList<>());
        for (Permission child : children) {
            result.add(child);
            getBrotherPermission(child.getId(), result, permissions);
        }
    }

    /**
     * 递归获取子权限
     *
     * @param pid         父权限id
     * @param permissions 所有权限集合
     * @return
     */
    private List<Permission> getChildrenPermissions(Long pid, List<Permission> permissions) {
        // 创建用于接收和返回递归权限数据的集合
        List<Permission> newPermissions = Lists.newArrayList();
        for (Permission permission : permissions) {
            // 判断当前权限是否为目标权限子权限
            if (null != permission.getParentId() && permission.getParentId().equals(pid)) {
                // 递归查询，获取子权限的子权限
                List<Permission> children = getChildrenPermissions(permission.getId(), permissions);
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermissionComparator());
                    permission.setChilds(children);
                } else {
                    permission.setChilds(null);
                }
                // 添加到集合中
                newPermissions.add(permission);
            }
        }
        newPermissions.sort(new PermissionComparator());
        return newPermissions;
    }

    /**
     * 公共方法，勿删除
     */
    @Override
    public GeneralResult<AccountSecurity> apiKey(String apiKey) {
        try {
            AccountSecurity security = securityRepository.getByApiKey(apiKey, null);
            return new GeneralResult<>(true, security, "获取安全密钥成功");
        } catch (Exception e) {
            log.error("get secKey fail:", e);
            return new GeneralResult<>(false, "获取安全密钥失败");
        }
    }

    /**
     * 批量添加收藏
     *
     * @param favorites
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result addFavorites(List<Favorite> favorites) throws InternalServerException {
        try {
            boolean flag = favoriteRepository.batchSave(favorites, 1000);
            return new Result(flag, flag ? "批量插入成功" : "批量插入失败");
        } catch (Exception e) {
            log.error("batch save favorite error,", e);
            throw new InternalServerException("批量插入失败", e, e.getMessage());
        }
    }

    /**
     * 添加收藏
     *
     * @param serviceId
     * @param tenantId
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result addFavorite(Long serviceId, Long tenantId) throws InternalServerException {
        try {
            // 创建收藏实体对象
            Favorite favorite = new Favorite(null, Favorite.Category.Tenant, tenantId, serviceId, 0);
            favoriteRepository.save(favorite);
            return new Result(true, "添加收藏成功");
        } catch (Exception e) {
            log.error("add favorite failed,", e);
            throw new InternalServerException("添加收藏失败", e, e.getMessage());
        }
    }

    /**
     * 取消收藏
     *
     * @param serviceId
     * @param tenantId
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result cancelFavorite(Long serviceId, Long tenantId, RequestContext requestContext)
            throws InternalServerException {
        try {
            favoriteRepository.cancelFavorite(serviceId, tenantId, RequestContext.Catalog.Tenant.name());
            return new Result(true, "取消收藏成功");
        } catch (Exception e) {
            log.error("remove favorite failed,", e);
            throw new InternalServerException("取消收藏失败", e, e.getMessage());
        }
    }

    /**
     * 获取服务目录
     *
     * @param requestContext
     * @return
     */
    @Override
    public GeneralResult<List<ServiceMenuBean>> listServiceMenu(RequestContext requestContext) {
        try {
            // 获取所有的服务菜单权限
            List<CloudServiceBean> allServiceBeans = cloudServiceBeanRepository.getServiceMenu();
            HashMap<String, List<CloudServiceBean>> serviceMenu = new HashMap<>(8);
            // 获取租户收藏的菜单权限
            List<CloudServiceBean> tenantService = cloudServiceBeanRepository
                    .getFavoriteServices(requestContext.getTarget(), RequestContext.Catalog.Tenant.name());
            List<Long> serviceIds = allServiceBeans.stream().map(CloudServiceBean::getId).collect(Collectors.toList());
            // 获取授权的服务对应的菜单权限
            List<Permission> servicePermissions = permissionRepository.getServicesPermissions(serviceIds);
            // 查询服务编码集合
            List<String> permissionCodes =
                    allServiceBeans.stream().map(CloudServiceBean::getCode).collect(Collectors.toList());
            HashMap<String, List<Permission>> servicePermissionMap = new HashMap<>(permissionCodes.size());
            for (String code : permissionCodes) {
                ArrayList<Permission> permissions = new ArrayList<>();
                for (Permission servicePermission : servicePermissions) {
                    if (servicePermission.getServiceCodes().contains(code)) {
                        permissions.add(servicePermission);
                    }
                }
                servicePermissionMap.put(code, permissions);
            }
            for (CloudServiceBean cloudServiceBean : allServiceBeans) {
                ArrayList<CloudServiceBean> menuList = new ArrayList<>();
                // 获取当前服务的菜单权限
                List<Permission> permissions = servicePermissionMap.get(cloudServiceBean.getCode());
                if (permissions.size() > 0) {
                    // 获取服务对应菜单的权限，并给服务菜单设置路由
                    Permission permission =
                            permissions.stream().filter(permit -> "isServiceMenu".equals(permit.getProps())).findAny()
                                    .orElse(permissions.get(0));
                    cloudServiceBean.setPath(permission.getPath());
                }
                if (tenantService.isEmpty()) {
                    // 如果租户收藏的菜单权限为空，订阅状态都设置为false
                    cloudServiceBean.setSubscribe(false);
                }
                for (CloudServiceBean service : tenantService) {
                    // 给租户已收藏的服务订阅状态设置为true
                    if (cloudServiceBean.getName().equals(service.getName())) {
                        cloudServiceBean.setSubscribe(true);
                        break;
                    } else {
                        cloudServiceBean.setSubscribe(false);
                    }
                }
                // 对服务菜单集合按服务类别进行分组
                if (!serviceMenu.containsKey(cloudServiceBean.getCatalogName())) {
                    menuList.add(cloudServiceBean);
                    serviceMenu.put(cloudServiceBean.getCatalogName(), menuList);
                } else {
                    List<CloudServiceBean> serviceMenus = serviceMenu.get(cloudServiceBean.getCatalogName());
                    serviceMenus.add(cloudServiceBean);
                    serviceMenu.put(cloudServiceBean.getCatalogName(), serviceMenus);
                }
            }
            ArrayList<ServiceMenuBean> serviceMenus = new ArrayList<>();
            ArrayList<String> category = Lists.newArrayList("计算", "企业云计算", "政务云计算", "存储", "网络", "数据库", "中间件", "运维", "编排", "监控");
            for (String key : category) {
                List<CloudServiceBean> serviceBeanList = serviceMenu.get(key);
                if (null != serviceBeanList && !serviceBeanList.isEmpty()) {
                    serviceMenus.add(new ServiceMenuBean(key, serviceBeanList));
                }
            }
            return new GeneralResult<>(true, serviceMenus, "查询服务目录成功");
        } catch (Exception e) {
            log.error("get service menus error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取收藏夹
     *
     * @param requestContext
     * @return
     */
    @Override
    public GeneralResult<List<CloudServiceBean>> listFavorite(RequestContext requestContext) {
        try {
            // 获取租户收藏的服务
            List<CloudServiceBean> favoriteServices = cloudServiceBeanRepository
                    .getFavoriteServices(requestContext.getTarget(), RequestContext.Catalog.Tenant.name());
            ArrayList<CloudServiceBean> favorites = new ArrayList<>();
            List<Long> serviceIds = favoriteServices.stream().map(CloudServiceBean::getId).collect(Collectors.toList());
            // 获取授权的服务对应的菜单权限
            List<Permission> servicePermissions = permissionRepository.getServicesPermissions(serviceIds);
            // 查询服务编码集合
            List<String> permissionCodes = new ArrayList<>();
            servicePermissions.forEach(permission -> {
                if (StringUtils.isNotBlank(permission.getServiceCodes())) {
                    permissionCodes.addAll(JSONArray.parseArray(permission.getServiceCodes(), String.class));
                }
            });
            HashMap<String, List<Permission>> servicePermissionMap = new HashMap<>(permissionCodes.size());
            for (String code : permissionCodes) {
                ArrayList<Permission> permissions = new ArrayList<>();
                for (Permission servicePermission : servicePermissions) {
                    if (servicePermission.getServiceCodes().contains(code)) {
                        permissions.add(servicePermission);
                    }
                }
                servicePermissionMap.put(code, permissions);
            }
            for (CloudServiceBean cloudServiceBean : favoriteServices) {
                List<Permission> permissions = servicePermissionMap.get(cloudServiceBean.getCode());
                if (permissions != null && permissions.size() > 0) {
                    // 获取当前服务对应的菜单权限，并给对应的服务目录对象设置路由
                    Permission permission =
                            permissions.stream().filter(permit -> "isServiceMenu".equals(permit.getProps())).findAny()
                                    .orElse(permissions.get(0));
                    cloudServiceBean.setPath(permission.getPath());
                }
                // 设置订阅状态
                cloudServiceBean.setSubscribe(true);
                // 设置排序优先级
                cloudServiceBean.setPriority(cloudServiceBean.getPriority());
                favorites.add(cloudServiceBean);
            }
            favorites.sort(new FavoriteComparator());
            return new GeneralResult<>(true, favorites, "查询服务收藏夹成功");
        } catch (Exception e) {
            log.error("get favorites error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 修改收藏夹排序
     *
     * @param priorityModels
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result changePriority(List<PriorityModel> priorityModels, RequestContext requestContext)
            throws InternalServerException {
        try {
            for (PriorityModel model : priorityModels) {
                tenantRepository.changePriority(model.getId(), model.getPriority(), requestContext.getTarget());
            }
            return new Result(true, "修改收藏夹排序成功");
        } catch (Exception e) {
            log.error("change favorite priority error,", e);
            throw new InternalServerException("修改收藏夹排序失败", e, e.getMessage());
        }
    }

    /**
     * 返回账号相关登录所需信息
     *
     * @param account
     * @return
     */
    @Override
    public GeneralResult<Map<String, Object>> tokenLogin(String account) {
        // 判断租户是否存在
        Tenant tenant = tenantRepository.getByAccount(account);
        Assert.notNull(tenant, "该账号存在问题,校验失败");
        Assert.isTrue(!tenant.getStatus().equals(BaseStatus.ABNORMAL.name()), "该账号用户已被冻结，用户无法登录");
        // 查询随机数
        AccountSecurity security = securityRepository.getByTarget(tenant.getId(), Catalog.Tenant);
        Assert.notNull(security, "该账号存在问题,校验失败");
        Map<String, Object> map = MapTools.simpleMap(Common.TENANT, tenant);
        map.put(Common.API_KEY, security.getApiKey());
        map.put(Common.SEC_KEY, security.getSecKey());
        map.put("accountCategory", Catalog.Tenant.name());
        return new GeneralResult<>(true, map, "租户跳转成功");
    }

    @Override
    public GeneralResult<List<Project>> projects(Long id) {
        List<Project> projects = projectRepository.listTenantProject(id);
        return new GeneralResult<>(true, projects, "查询成功");
    }

    @Override
    public GeneralResult<List<User>> getTenantUsers(Long id) {
        List<User> users = userRepository.listByTenant(id);
        return new GeneralResult<>(true, users, "查询成功");
    }

    @Override
    public Result importTemp(String name, RequestContext requestContext, HttpServletResponse response) {
        List<String> departmentNames = new ArrayList<>();
        List<String> isQuota = new ArrayList<>();
        try {
            departmentNames =
                    departmentRepository.listAll().stream().map(Department::getName).collect(Collectors.toList());
        } catch (Exception e) {
            log.error("获取部门数据异常", e);
        }
        isQuota.add("是");
        isQuota.add("否");
        HashMap<Integer, List<String>> map = new HashMap<>(8);
        if (departmentNames.size() > 0) {
            map.put(3, departmentNames);
        }
        map.put(9, isQuota);
        CellWriteHandler cellWriteHandler = getCellWriteHandler(map);
        boolean exportExcel =
                exportExcel(name, "导入模板", response, new ArrayList<>(), ImportTenantTemplate.class, cellWriteHandler);
        return exportExcel ? new Result(true, "租户模板下载成功") : new Result(false, "租户模板下载失败");
    }

    @Override
    public Result importData(MultipartFile multipartFile, RequestContext requestContext) {
        String originalFilename = multipartFile.getOriginalFilename();
        Assert.notNull(originalFilename, "文件名不能为空");
        String[] CheckedHead =
                new String[]{"租户名称", "登录账号", "集团编码", "所属组织", "联系人姓名", "联系人邮箱", "联系人电话", "折扣率", "租户描述", "是否开启配额"};
        AnalysisEventListener<ImportTenantTemplate> analysisEventListener = ImportExcelUtil
                .getListener(this.batchInsert(), 1000, CheckedHead, MapTools.simpleMap("requestContext", requestContext));
        try {
            EasyExcel.read(multipartFile.getInputStream(), ImportTenantTemplate.class, analysisEventListener).sheet()
                    .doRead();
        } catch (IOException e) {
            log.error("读取租户列表异常", e);
            return new Result(false, "导入租户列表异常");
        }
        return ImportExcelUtil.getResult();
    }

    private BiFunction<List<ImportTenantTemplate>, Map<String, Object>, Result> batchInsert() {
        return (importTenantTemplates, paramsMap) -> {
            RequestContext requestContext = (RequestContext) paramsMap.get("requestContext");
            ArrayList<Tenant> tenants = new ArrayList<>();
            for (ImportTenantTemplate impTenantTemp : importTenantTemplates) {
                // 防止导入空行
                if (impTenantTemp.getName() == null && impTenantTemp.getAccount() == null
                        && impTenantTemp.getContactName() == null) {
                    continue;
                }
                Tenant tenant = new Tenant();
                tenant.setName(impTenantTemp.getName());
                tenant.setAccount(impTenantTemp.getAccount());
                tenant.setGroupCode(impTenantTemp.getGroupCode());
                tenant.setDepartmentName(impTenantTemp.getDepartmentName());
                tenant.setContactEmail(impTenantTemp.getEmail());
                tenant.setDiscount(impTenantTemp.getDiscount());
                tenant.setQuotaAble("是".equals(impTenantTemp.getIsQuota()));
                tenant.setContactName(impTenantTemp.getContactName());
                tenant.setContactPhone(impTenantTemp.getContactPhone());
                tenant.setRemark(impTenantTemp.getRemark());
                tenants.add(tenant);
            }
            // 错误信息
            StringBuilder errorMsg = new StringBuilder();
            // 按照错误类型分组存储错误信息
            StringBuilder nullValueMsg = new StringBuilder();
            StringBuilder emailFormatMsg = new StringBuilder();
            StringBuilder groupFormatMsg = new StringBuilder();
            StringBuilder nameExistsMsg = new StringBuilder();
            StringBuilder depNotExistsMsg = new StringBuilder();
            StringBuilder emailUsedMsg = new StringBuilder();
            StringBuilder accountUsedMsg = new StringBuilder();
            StringBuilder codeUsed = new StringBuilder();
            try {
                int successNum = 0;
                int errorNumber = 0;
                // 邮箱格式
                String EMAIL_REGEX = "^[\\w-_\\.+]*[\\w-_\\.]\\@([\\w]+\\.)+[\\w]+[\\w]$";
                String CODE_REGEX = "^[A-Za-z0-9]+$";
                List<String> existNames =
                        tenantRepository.list().stream().map(Tenant::getName).collect(Collectors.toList());
                Map<String, Department> departmentMap = departmentRepository.listAll().stream()
                        .collect(Collectors.toMap(Department::getName, Function.identity()));
                ListIterator<Tenant> listIterator = tenants.listIterator();
                while (listIterator.hasNext()) {
                    Tenant tenant = listIterator.next();
                    if (tenant.getAccount() == null || tenant.getName() == null) {
                        errorNumber++;
                        log.error("{}租户名称或账号为空", tenant.getName());
                        nullValueMsg.append(tenant.getName()).append(",");
                        listIterator.remove();
                        continue;
                    }
                    String groupCode = tenant.getGroupCode();
                    // 租户邮箱格式校验
                    if (groupCode != null && !groupCode.matches(CODE_REGEX)) {
                        errorNumber++;
                        log.error("{}集团编码格式不正确", tenant.getName());
                        groupFormatMsg.append(tenant.getName()).append(",");
                        listIterator.remove();
                        continue;
                    }

                    String email = tenant.getContactEmail();
                    // 租户邮箱格式校验
                    if (email != null && !email.matches(EMAIL_REGEX)) {
                        errorNumber++;
                        log.error("{}邮箱格式错误", tenant.getName());
                        emailFormatMsg.append(tenant.getName()).append(",");
                        listIterator.remove();
                        continue;
                    }
                    String departmentName = tenant.getDepartmentName();
                    if (existNames.contains(tenant.getName())) {
                        errorNumber++;
                        listIterator.remove();
                        nameExistsMsg.append(tenant.getName()).append(",");
                        log.error("tenant {}  has existed", tenant.getName());
                        continue;
                    }
                    if (StringUtils.isNotBlank(departmentName)) {
                        if (!departmentMap.containsKey(departmentName)) {
                            errorNumber++;
                            listIterator.remove();
                            depNotExistsMsg.append(tenant.getName()).append(",");
                            log.error("tenant {} 's department {} don't exist", tenant.getName(), departmentName);
                        } else {
                            Department department = departmentMap.get(departmentName);
                            tenant.setDepartId(department.getId());
                        }
                    }
                }
                ArrayList<GeneralResult<Map<String, StringBuilder>>> list = new ArrayList<>();
                for (Tenant tenant : tenants) {
                    Future<GeneralResult<Map<String, StringBuilder>>> resultFuture =
                            taskExecutor.submit(() -> this.createTenant(tenant, requestContext));
                    if (!JSONObject.parseObject(JSON.toJSONString(resultFuture.get())).getBoolean("success")) {
                        errorNumber++;
                    }
                    GeneralResult<Map<String, StringBuilder>> createTenantResult = resultFuture.get();
                    list.add(createTenantResult);
                }
                for (GeneralResult<Map<String, StringBuilder>> res : list) {
                    if (res.isSuccess()) {
                        successNum++;
                    } else {
                        Map<String, StringBuilder> msg = res.getData();
                        if (msg != null) {
                            if (msg.get("emailUsed") != null && msg.get("emailUsed").length() > 0) {
                                emailUsedMsg.append(msg.get("emailUsed"));
                            }
                            if (msg.get("accountUsed") != null && msg.get("accountUsed").length() > 0) {
                                accountUsedMsg.append(msg.get("accountUsed"));
                            }
                            if (msg.get("nameUsed") != null && msg.get("nameUsed").length() > 0) {
                                nameExistsMsg.append(msg.get("nameUsed"));
                            }
                            if (msg.get("codeUsed") != null && msg.get("codeUsed").length() > 0) {
                                codeUsed.append(msg.get("codeUsed"));
                            }
                        }
                    }
                }
                if (successNum == tenants.size() && errorNumber == 0) {
                    return new Result(true, "导入租户信息成功");
                } else {
                    if (nullValueMsg.length() > 0) {
                        errorMsg.append("<p>").append("租户名称或账号为空：").append(nullValueMsg).append("</p>");
                    }
                    if (emailFormatMsg.length() > 0) {
                        errorMsg.append("<p>").append("邮箱格式不正确：").append(emailFormatMsg).append("</p>");
                    }
                    if (groupFormatMsg.length() > 0) {
                        errorMsg.append("<p>").append("集团编码格式不正确：").append(groupFormatMsg).append("</p>");
                    }
                    if (nameExistsMsg.length() > 0) {
                        errorMsg.append("<p>").append("租户名称已存在：").append(nameExistsMsg).append("</p>");
                    }
                    if (depNotExistsMsg.length() > 0) {
                        errorMsg.append("<p>").append("部门不存在：").append(depNotExistsMsg).append("</p>");
                    }
                    if (emailUsedMsg.length() > 0) {
                        errorMsg.append("<p>").append("邮箱已使用：").append(emailUsedMsg).append("</p>");
                    }
                    if (accountUsedMsg.length() > 0) {
                        errorMsg.append("<p>").append("账号已使用：").append(accountUsedMsg).append("</p>");
                    }
                    if (codeUsed.length() > 0) {
                        errorMsg.append("<p>").append("租户编码已使用：").append(codeUsed).append("</p>");
                    }
                    return new Result(false, successNum + "条租户信息导入成功，" + errorNumber + "条租户信息导入失败 " + errorMsg);
                }
            } catch (Exception e) {
                log.error("导入租户异常", e);
                return new Result(false, "导入租户信息异常" + errorMsg);
            }
        };
    }

    private GeneralResult<List<Tenant>> readExcel(Workbook wb) {
        Sheet sheet = wb.getSheetAt(0);
        List<Tenant> tenants = new ArrayList<>();
        for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
            Row row = sheet.getRow(rowNum);
            // 模板校验
            if (rowNum == 0) {
                if (!templateCheck(row)) {
                    log.error("导入租户格式不正确");
                    return new GeneralResult<>(false, null, "导入租户格式不正确");
                }
                continue;
            }
            Tenant tenant = new Tenant();
            Cell name = row.getCell(0);
            tenant.setName(getCellStringValue(name));
            Cell account = row.getCell(1);
            tenant.setAccount(getCellStringValue(account));
            Cell code = row.getCell(2);
            // tenant.setCode(getCellStringValue(code));
            tenant.setGroupCode(getCellStringValue(code));
            Cell depart = row.getCell(3);
            tenant.setDepartmentName(getCellStringValue(depart));
            Cell contactName = row.getCell(4);
            tenant.setContactName(getCellStringValue(contactName));
            Cell contactEmail = row.getCell(5);
            tenant.setContactEmail(getCellStringValue(contactEmail));
            Cell contactPhone = row.getCell(6);
            tenant.setContactPhone(getCellStringValue(contactPhone));
            Cell discount = row.getCell(7);
            tenant.setDiscount(BigDecimal.valueOf(Double.parseDouble(
                    StringUtils.isNotBlank(getCellStringValue(discount)) ? getCellStringValue(discount) : "1")));
            Cell remark = row.getCell(8);
            tenant.setRemark(getCellStringValue(remark));
            Cell quotaAble = row.getCell(9);
            tenant.setQuotaAble("是".equals(getCellStringValue(quotaAble)));
            tenants.add(tenant);
        }
        return new GeneralResult<>(true, tenants, "读取租户成功");
    }

    // 租户导入模板校验
    private boolean templateCheck(Row row) {
        String[] cellTitle =
                new String[]{"租户名称", "登录账号", "集团编码", "所属组织", "联系人姓名", "联系人邮箱", "联系人电话", "折扣率", "租户描述", "是否开启配额"};
        for (int i = 0; i < cellTitle.length; i++) {
            if (!cellTitle[i].equals(getCellStringValue(row.getCell(i)))) {
                return false;
            }
        }
        return true;
    }

    private GeneralResult<Map<String, StringBuilder>> createTenant(Tenant tenant, RequestContext requestContext) {
        // 重复
        StringBuilder emailUsed = new StringBuilder();
        StringBuilder accountUsed = new StringBuilder();
        StringBuilder nameUsed = new StringBuilder();
        StringBuilder codeUsed = new StringBuilder();
        StringBuilder errorMsg = new StringBuilder();

        try {
            tenant.setCreatorId(requestContext.getTarget());
            // 初始化密码
            String salt = UUID.randomUUID().toString();
            Encryptor encryptor = new SHAEncryptor();
            // 创建保存租户基本信息
            Tenant oldTenant = tenantRepository.getByEmail(tenant.getEmail());
            if (oldTenant != null) {
                emailUsed.append(oldTenant.getName()).append(",");
            }
            Assert.isNull(oldTenant, "邮箱已被注册");
            Tenant existAccount = tenantRepository.getByAccount(tenant.getAccount());
            if (existAccount != null) {
                accountUsed.append(existAccount.getName()).append(",");
            }
            Assert.isNull(existAccount, "该账号已存在");
            Tenant existName = tenantRepository.getByName(tenant.getName());
            if (existName != null) {
                nameUsed.append(existName.getName()).append(",");
            }
            Assert.isNull(existName, "该租户已存在");
            Tenant codeTenant = tenantRepository.getByCode(tenant.getGroupCode());
            if (codeTenant != null) {
                codeUsed.append(codeTenant.getName()).append(",");
            }
            Assert.isNull(tenantRepository.getByCode(tenant.getGroupCode()), "该租户编码已存在");
            String password = encryptor.encrypt("Aa123456", salt);
            tenant.setPassword(password);
            // 关键信息去空
            tenant.setAccount(tenant.getAccount().trim());
            tenant.setName(tenant.getName().trim());
            tenant.setStatus(BaseStatus.NORMAL.name());
            tenant.setLoginStatus(false);
            tenant.setLastPwdModifyDate(new Date());
            tenantRepository.save(tenant);
            String account = tenant.getAccount();
            Tenant cacheTenant = tenantRepository.getByAccount(account);
            // 添加租户信息缓存
            redisTemplate.opsForValue().set(Catalog.Tenant.name() + "_" + cacheTenant.getId(), account);
            redisTemplate.opsForValue().set(Catalog.Tenant.name() + "_Name_" + cacheTenant.getId(),
                    cacheTenant.getName());
            // 构建租户随机数对象
            AccountSecurity security = new AccountSecurity();
            String data = tenant.getAccount() + System.currentTimeMillis();
            String apiKey = encryptor.encrypt(data, Common.API_KEY);
            String secKey = encryptor.encrypt(data, Common.SEC_KEY);
            security.setTargetId(tenant.getId());
            security.setCategory(AccountSecurity.Category.Tenant);
            security.setSalt(salt);
            security.setApiKey(apiKey);
            security.setSecKey(secKey);
            security.setTenantId(tenant.getId());
            boolean success = securityRepository.baseSave(security);
            if (!success) {
                this.callback(tenant.getId(), requestContext);
            }
            HashMap<String, StringBuilder> map = new HashMap<>();
            if (emailUsed.length() > 0) {
                map.put("emailUsed", emailUsed);
                errorMsg.append(emailUsed);
            }
            if (emailUsed.length() > 0) {
                map.put("accountUsed", accountUsed);
                errorMsg.append(accountUsed);
            }
            if (emailUsed.length() > 0) {
                map.put("nameUsed", nameUsed);
                errorMsg.append(nameUsed);
            }
            if (emailUsed.length() > 0) {
                map.put("codeUsed", codeUsed);
                errorMsg.append(codeUsed);
            }
            try {
                // 创建租户授信账号对象
                TenantAccount tenantAccount = new TenantAccount(tenant.getId(), BigDecimal.ZERO, BigDecimal.ZERO,
                        BigDecimal.ZERO, BigDecimal.ZERO, true);
                // 获取授权服务
                List<Long> serviceIds = new ArrayList<>();
                if (ListTool.hasElement(tenant.getServices())) {
                    serviceIds = tenant.getServices().stream().map(ServiceBean::getServiceId).collect(Collectors.toList());
                }
                TenantGrantMessage tenantGrantMessage =
                        new TenantGrantMessage(tenant.getId(), serviceIds, tenantAccount, requestContext);
                // 添加收藏夹
                List<Favorite> favorites = new ArrayList<>();
                for (Long serviceId : serviceIds) {
                    Favorite favorite = new Favorite(null, Favorite.Category.Tenant, tenant.getId(), serviceId, 0);
                    favorites.add(favorite);
                }
                if (!favorites.isEmpty()) {
                    for (Favorite favorite : favorites) {
                        favoriteRepository.save(favorite);
                    }
                }
            } catch (Exception e) {
                log.error("Create Tenant fail:", e);
                throw new InternalServerException("租户创建失败", e, e.getMessage());
            }


            return new GeneralResult<>(success, map, "");
        } catch (Exception e) {
            log.error("create tenant fail:", e);
            this.callback(tenant.getId(), requestContext);
            return new GeneralResult<>(false, errorMsg.toString());
        }


    }

    @Override
    public Result export(List<Param> paramList, RequestContext requestContext, HttpServletResponse response) {

        ArrayList<ImportTenantTemplate> excelModeList = null;
        try {
            Map<String, String> sorter = new HashMap<>(8);
            sorter.put("gmtCreate", Common.ONE);
            List<Tenant> tenants = tenantRepository.list(1, Integer.MAX_VALUE, paramList, sorter);
            excelModeList = new ArrayList<>();
            for (Tenant tenant : tenants) {
                Department department = departmentRepository.query(tenant.getDepartId());
                if (department != null) {
                    tenant.setDepartmentName(department.getName());
                }
                ImportTenantTemplate importTenant = new ImportTenantTemplate();
                importTenant.setName(tenant.getName());
                importTenant.setAccount(tenant.getAccount());
                importTenant.setDepartmentName(tenant.getDepartmentName());
                importTenant.setContactName(tenant.getContactName());
                importTenant.setContactPhone(tenant.getContactPhone());
                importTenant.setDiscount(tenant.getDiscount());
                importTenant.setEmail(tenant.getEmail());
                importTenant.setGroupCode(tenant.getGroupCode());
                importTenant.setIsQuota(tenant.getQuotaAble() ? "是" : "否");
                importTenant.setRemark(tenant.getRemark());
                excelModeList.add(importTenant);
            }
        } catch (Exception e) {
            log.error("查询租户异常", e);
            return new Result(false, "查询租户异常");
        }
        boolean res = ExportUtil.exportExcel("租户列表" + System.currentTimeMillis(), "租户列表", response, excelModeList,
                ImportTenantTemplate.class, null);
        return res ? new Result(true, "") : new Result(false, "租户列表导出异常");
    }

    @Override
    public Result configUsers(Long id, List<Long> userIds) {
        try {
            List<User> updateUsers = new ArrayList<>();
            List<AccountSecurity> updateSecurity = new ArrayList<>();
            List<User> oldTenantUsers = userRepository.listByTenant(id);
            List<User> removeUsers =
                    oldTenantUsers.stream().filter(user -> !userIds.contains(user.getId())).toList();
            for (User removeUser : removeUsers) {
                removeUser.setTenantId(0L);
                updateUsers.add(removeUser);
                AccountSecurity security = securityRepository.getByTarget(removeUser.getId(), Catalog.User);
                security.setTenantId(null);
                updateSecurity.add(security);
            }
            userIds.add(-1L);
            List<User> users = userRepository.listByIds(userIds);
            for (User user : users) {
                user.setTenantId(id);
                updateUsers.add(user);
                AccountSecurity security = securityRepository.getByTarget(user.getId(), Catalog.User);
                security.setTenantId(id);
                updateSecurity.add(security);
            }
            if (!updateUsers.isEmpty()) {
                userRepository.batchUpdate(updateUsers, updateUsers.size());
            }
            if (!updateSecurity.isEmpty()) {
                securityRepository.batchUpdate(updateSecurity, updateSecurity.size());
            }
            return new Result(true, "租户配置用户成功");
        } catch (Exception e) {
            log.error("tenant config user fail:", e);
            return new Result(false, "租户配置用户失败");
        }
    }

    @Override
    public GeneralResult<?> condition(String condition) {
        try {
            JSONObject object = JSONObject.parseObject(condition);
            if ("checkPoolGroup".equals(object.getString("condition"))) {
                return this.checkPoolGroup(object.getLong("vendorId"));
            }
            return new GeneralResult<>(false, "不支持的查询");
        } catch (Exception e) {
            log.error("check pool group error", e);
            return GeneralResult.FAILED("查询失败");
        }
    }

    private GeneralResult<?> checkPoolGroup(Long vendorId) {
        try {
            boolean check = tenantPoolRepository.checkPoolGroup(vendorId);
            return new GeneralResult<>(true, check, "查询成功");
        } catch (Exception e) {
            log.error("check pool group error", e);
            return GeneralResult.FAILED("查询失败");
        }
    }
}
