package com.bocloud.sms.repository;

import com.bocloud.sms.entity.Permission;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author tyl
 * @time 2020/2/3 周一 11:43
 */
@Repository("permissionRepository")
public class PermissionRepository extends JdbcGenericDao<Permission, Long> {

    @Value("${spring.datasource.cos:cos}")
    private String cosDb;

    @Autowired
    public PermissionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据父类ID查询
     *
     * @param parentId
     * @param tenant
     * @return
     * @throws DataBaseException
     */
    public List<Permission> list(Long parentId, Boolean tenant) throws DataBaseException {
        if (null == parentId || parentId == 0) {
            String sql = "select * from permission where (parent_id = 0 or parent_id is null) and is_deleted = 0 and is_tenant = :tenant";
            Map<String, Object> paramMap = MapTools.simpleMap("tenant", tenant);
            return this.list(Permission.class, sql, paramMap);
        } else {
            String sql = "select * from permission where parent_id = :parentId and is_deleted = 0 and is_tenant = :tenant";
            Map<String, Object> params = new HashMap<>(2);
            params.put("parentId", parentId);
            params.put("tenant", tenant);
            return this.list(Permission.class, sql, params);
        }
    }

    /**
     * 查询所有权限
     *
     * @param tenant 是否是租户权限
     * @return
     * @throws DataBaseException
     */
    public List<Permission> list(Boolean tenant) throws DataBaseException {
        String sql = "select * from permission where is_deleted = 0 and is_tenant = :tenant";
        Map<String, Object> paramMap = MapTools.simpleMap("tenant", tenant);
        return this.list(Permission.class, sql, paramMap);
    }

    /**
     * 查询跟节点列表
     *
     * @param tenant
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listRoots(Boolean tenant) throws DataBaseException {
        String sql = "select * from permission where parent_id = 0 and is_deleted = 0 and is_tenant = :tenant";
        Map<String, Object> paramMap = MapTools.simpleMap("tenant", tenant);
        return this.list(Permission.class, sql, paramMap);
    }

    /**
     * 查询权限列表
     *
     * @param tenant
     * @param category
     * @param parentId
     * @return
     * @throws DataBaseException
     */
    public List<Permission> list(Boolean tenant, String category, Long parentId) throws DataBaseException {
        String sql = "select * from permission where is_deleted = 0 ";
        Map<String, Object> paramMap = new HashMap<>(16);
        if (tenant != null) {
            sql += " and is_tenant = :tenant ";
            paramMap.put("tenant", tenant);
        }
        if (!StringUtils.hasText(category)) {
            sql += " and category in ('menu','view')";
        } else {
            sql += " and category = :category";
            paramMap.put("category", category);
        }
        if (null != parentId && 0L != parentId) {
            sql += " and parent_id = :parentId";
            paramMap.put("parentId", parentId);
        }
        return this.list(Permission.class, sql, paramMap);
    }

    /**
     * 根据角色ID查询权限
     *
     * @param roleId
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listByRole(Long roleId) throws DataBaseException {
        String sql = "select p.* from permission p left join role_permission rp on rp.permission_id = p.id where rp.role_id = :roleId";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        return this.list(Permission.class, sql, params);
    }

    /**
     * 根据权限类型查询
     *
     * @param roleId
     * @param category
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listByCategory(Long roleId, String category) throws DataBaseException {
        String sql = "select a.* from permission a , role_permission b where b.role_id= :roleId and a.id = b.permission_id and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("roleId", roleId);
        if (!StringUtils.hasText(category)) {
            sql += " and a.category in ('menu','view')";
        } else {
            sql += " and a.category = :category";
            params.put("category", category);
        }
        return this.list(Permission.class, sql, params);
    }

    /**
     * 删除权限
     *
     * @param id
     * @param menderId
     * @return
     * @throws DataBaseException
     */
    public int delete(Long id, Long menderId) throws DataBaseException {
        String sql = "update permission set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = new HashMap<>(3);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("id", id);
        return this.execute(sql, params);
    }


    /**
     * 删除权限
     *
     * @param ids
     * @param menderId
     * @return
     * @throws DataBaseException
     */
    public int batchDelete(List<Long> ids, Long menderId) throws DataBaseException {
        String sql = "update permission set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id in (:ids)";
        Map<String, Object> params = new HashMap<>(3);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("ids", ids);
        return this.execute(sql, params);
    }

    /**
     * 根据ID查询
     *
     * @param id
     * @return
     * @throws DataBaseException
     */
    public Permission query(Long id) throws DataBaseException {
        String sql = "select * from permission where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Permission> list = this.list(Permission.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one permission in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据租户ID查询
     *
     * @param tenantId
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listByTenant(Long tenantId) throws DataBaseException {
        String sql = "select a.* from permission a left join tenant_permission b on a.id = b.permission_id where a.is_deleted = 0 ";
        Map<String, Object> params = null;
        if (tenantId != null && tenantId != 0L) {
            sql += "and b.tenant_id = :tenantId";
            params = MapTools.simpleMap("tenantId", tenantId);
        }
        return this.list(Permission.class, sql, params);
    }

    /**
     * 根据权限类型查询
     *
     * @param category
     * @param parentId
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listByCategory(String category, Long parentId) throws DataBaseException {
        String sql = "select a.* from permission a where a.is_deleted = 0 and a.category= :category ";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        if (null != parentId && 0L != parentId) {
            sql += " and a.parent_id = :parentId";
            params.put("parentId", parentId);
        }
        return this.list(Permission.class, sql, params);
    }

    /**
     * 查询用户授权权限（包括父级）
     *
     * @param userId
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listByUser(Long userId) throws DataBaseException {
        String sql = "select DISTINCT a.* from permission a left join role_permission b on b.permission_id = a.id"
                + " left join user_role c on c.role_id = b.role_id where c.user_id = :userId";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        return this.list(Permission.class, sql, params);
    }

    /**
     * 查询所有权限
     *
     * @return
     * @throws DataBaseException
     */
    public List<Permission> listAll() throws DataBaseException {
        String sql = "SELECT * from permission where is_deleted = 0 ";
        return this.list(Permission.class, sql);
    }

    public List<Permission> listByName(String name) throws DataBaseException {
        String sql = "SELECT * from permission where is_deleted = 0 and name = :name ";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        return this.list(Permission.class, sql, params);
    }

    public List<Permission> getServicePermissions(Long serviceId) throws DataBaseException {
        String sql = "select p.* from permission p left join " + cosDb + ".cloud_service cs on locate(cs.code, p.service_codes)  where p.is_deleted = 0 and cs.id = :serviceId";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        return this.list(Permission.class, sql, params);
    }

    public List<Permission> getServicesPermissions(List<Long> serviceIds) throws DataBaseException {
        String sql = "select distinct p.* from permission p left join " + cosDb + ".cloud_service cs on locate(cs.code, p.service_codes)  where p.is_deleted = 0 and cs.id in (:serviceIds)";
        Map<String, Object> params = MapTools.simpleMap("serviceIds", serviceIds);
        return !serviceIds.isEmpty() ? this.list(Permission.class, sql, params) : new ArrayList<>();
    }

    /**
     * 插件绑定权限
     * 插件绑定权限表时，需要将插件唯一编码 和 插件状态同步绑定过去
     *
     * @param permissionIdList 权限id列表
     * @param pluginsCode      插件编码
     * @param pluginsStatus    插件状态
     * @return
     */
    public int bindingPlugins(List<Long> permissionIdList, String pluginsCode, Boolean pluginsStatus) {
        String sql = "UPDATE permission SET plugins_code = :pluginsCode,plugins_status = :pluginsStatus WHERE is_deleted = 0 AND id in (:permissionIdList)";
        Map<String, Object> params = MapTools.simpleMap("pluginsStatus", pluginsStatus);
        params.put("pluginsCode", pluginsCode);
        params.put("permissionIdList", permissionIdList);
        return this.execute(sql, params);
    }

    /**
     * 解绑操作 解绑该插件下所有绑定的菜单列表
     * 解绑时将权限表中的插件名称改为"" 插件状态改为关闭
     *
     * @param pluginsCode   插件编码
     * @param pluginsStatus 插件状态
     * @param tenant        是否是租户端菜单
     * @return
     */
    public int unBindingPlugins(String pluginsCode, Boolean pluginsStatus, Boolean tenant) {
        String sql = "UPDATE permission SET plugins_code = '',plugins_status = :pluginsStatus WHERE is_deleted = 0 AND plugins_code = :pluginsCode";
        Map<String, Object> params = MapTools.simpleMap("pluginsCode", pluginsCode);
        params.put("pluginsStatus", pluginsStatus);
        if (tenant != null) {
            sql += " AND is_tenant = :tenant ";
            params.put("tenant", tenant);
        }
        return this.execute(sql, params);
    }

    /**
     * 根据插件编码，修改权限表中的plugin_code
     *
     * @param pluginsCode   插件编码
     * @param pluginsStatus 插件状态
     * @return
     */
    public int updateByPluginsStatus(String pluginsCode, Boolean pluginsStatus) {
        String sql = "UPDATE permission SET plugins_status = :pluginsStatus WHERE is_deleted = 0 AND plugins_code = :pluginsCode";
        Map<String, Object> params = MapTools.simpleMap("pluginsStatus", pluginsStatus);
        params.put("pluginsCode", pluginsCode);
        return this.execute(sql, params);
    }

    /**
     * 根据插件编码，修改权限表中的plugin_code
     *
     * @param newCode
     * @param oldCode
     * @return
     */
    public int updateByPluginsCode(String newCode, String oldCode) {
        String sql = "update permission set plugins_code = :newCode where plugins_code = :oldCode and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("newCode", newCode);
        params.put("oldCode", oldCode);
        return this.execute(sql, params);
    }

    /**
     * 根据parentId查询
     *
     * @param parentId
     * @return
     * @throws DataBaseException
     */
    public Permission queryByParentId(Long parentId, String name) {
        String sql = "select * from permission where is_deleted = 0 and parent_id = :parentId ";
        Map<String, Object> params = MapTools.simpleMap("parentId", parentId);
        if (name != null) {
            sql += " and name = :name ";
            params.put("name", name);
        }
        List<Permission> list = this.list(Permission.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one permission in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询管理员授权权限
     *
     * @param managerId
     * @return
     * @throws Exception
     */
    public List<Permission> getPermissions(Long managerId) {
        String sql = "SELECT aa.* FROM permission aa JOIN " +
                "(SELECT DISTINCT b.permission_id FROM manager_role a " +
                " JOIN role_permission b ON a.role_id = b.role_id " +
                " WHERE a.manager_id = :managerId) bb ON aa.id = bb.permission_id where aa.is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("managerId", managerId);
        return Optional.ofNullable(super.list(Permission.class, sql, params)).orElse(new ArrayList<>());
    }

    /**
     * 获取当前用户按钮权限
     *
     * @param context 当前用户
     * @return 按钮权限集合
     */
    public List<Permission> listButtonPermissions(RequestContext context) {
        String sql = " select distinct p.* from permission p left join role_permission rp on p.id = rp.permission_id " +
                " left join role r on r.id = rp.role_id and r.is_deleted = 0 and r.status = 'NORMAL' " +
                " left join manager_role mr on mr.role_id = r.id " +
                " left join users u on mr.manager_id = u.id and u.is_deleted = 0 " +
                " where p.is_deleted = 0 and p.is_tenant = :isTenant and p.category = :category and u.id = :userId ";
        Map<String, Object> params = new HashMap<>(10);
        params.put("isTenant", false);
        params.put("category", "button");
        params.put("userId", context.getTarget());
        return super.list(Permission.class, sql, params);
    }

    /**
     * 获取租户角色按钮权限
     *
     * @param context 当前用户
     * @return 按钮权限集合
     */
    public List<Permission> listTenantButtonPermissions(RequestContext context) {
        String sql = " select distinct p.* from permission p left join role_permission rp on p.id = rp.permission_id " +
                " left join role r on r.id = rp.role_id and r.is_deleted = 0 and r.status = 'NORMAL' " +
                " where p.is_deleted = 0 and p.is_tenant = :isTenant and p.category = :category and r.props = :props ";
        Map<String, Object> params = new HashMap<>(10);
        params.put("isTenant", true);
        params.put("category", "button");
        params.put("props", RequestContext.Catalog.Tenant == context.getCatalog() ? "tenantAdmin" : "tenantUser");
        return super.list(Permission.class, sql, params);
    }
}
